@@ -7,6 +7,7 @@ import time
 from typing import Any
 from typing import Callable
 from typing import cast
+from typing import Container
 from typing import Dict
 from typing import Generator
 from typing import Iterator
@@ -17,11 +18,9 @@ from typing import Union
 import warnings
 
 import numpy as np
-from packaging import version
 import tqdm
 
 import optuna
-from optuna._deprecated import deprecated
 from optuna._imports import try_import
 from optuna.integration._lightgbm_tuner.alias import _handling_alias_metrics
 from optuna.integration._lightgbm_tuner.alias import _handling_alias_parameters
@@ -62,7 +61,7 @@ _DEFAULT_LIGHTGBM_PARAMETERS = {
 _logger = optuna.logging.get_logger(__name__)
 
 
-class _BaseTuner(object):
+class _BaseTuner:
     def __init__(
         self,
         lgbm_params: Optional[Dict[str, Any]] = None,
@@ -211,7 +210,7 @@ class _OptunaObjective(_BaseTuner):
             self.lgbm_params["lambda_l2"] = trial.suggest_float("lambda_l2", 1e-8, 10.0, log=True)
         if "num_leaves" in self.target_param_names:
             tree_depth = self.lgbm_params.get("max_depth", _DEFAULT_TUNER_TREE_DEPTH)
-            max_num_leaves = 2 ** tree_depth if tree_depth > 0 else 2 ** _DEFAULT_TUNER_TREE_DEPTH
+            max_num_leaves = 2**tree_depth if tree_depth > 0 else 2**_DEFAULT_TUNER_TREE_DEPTH
             self.lgbm_params["num_leaves"] = trial.suggest_int("num_leaves", 2, max_num_leaves)
         if "feature_fraction" in self.target_param_names:
             # `GridSampler` is used for sampling feature_fraction value.
@@ -273,10 +272,14 @@ class _OptunaObjective(_BaseTuner):
             self.pbar.set_description(self.pbar_fmt.format(self.step_name, self.best_score))
             self.pbar.update(1)
 
-        trial.set_system_attr(_ELAPSED_SECS_KEY, elapsed_secs)
-        trial.set_system_attr(_AVERAGE_ITERATION_TIME_KEY, average_iteration_time)
-        trial.set_system_attr(_STEP_NAME_KEY, self.step_name)
-        trial.set_system_attr(_LGBM_PARAMS_KEY, json.dumps(self.lgbm_params))
+        trial.storage.set_trial_system_attr(trial._trial_id, _ELAPSED_SECS_KEY, elapsed_secs)
+        trial.storage.set_trial_system_attr(
+            trial._trial_id, _AVERAGE_ITERATION_TIME_KEY, average_iteration_time
+        )
+        trial.storage.set_trial_system_attr(trial._trial_id, _STEP_NAME_KEY, self.step_name)
+        trial.storage.set_trial_system_attr(
+            trial._trial_id, _LGBM_PARAMS_KEY, json.dumps(self.lgbm_params)
+        )
 
         self.trial_count += 1
 
@@ -361,7 +364,7 @@ class _LightGBMBaseTuner(_BaseTuner):
         feature_name: str = "auto",
         categorical_feature: str = "auto",
         early_stopping_rounds: Optional[int] = None,
-        verbose_eval: Optional[Union[bool, int]] = True,
+        verbose_eval: Optional[Union[bool, int, str]] = None,
         callbacks: Optional[List[Callable[..., Any]]] = None,
         time_budget: Optional[int] = None,
         sample_size: Optional[int] = None,
@@ -698,7 +701,7 @@ class _LightGBMBaseTuner(_BaseTuner):
             def get_trials(
                 self,
                 deepcopy: bool = True,
-                states: Optional[Tuple[TrialState, ...]] = None,
+                states: Optional[Container[TrialState]] = None,
             ) -> List[optuna.trial.FrozenTrial]:
 
                 trials = super().get_trials(deepcopy=deepcopy, states=states)
@@ -758,16 +761,16 @@ class LightGBMTuner(_LightGBMBaseTuner):
         optuna_callbacks:
             List of Optuna callback functions that are invoked at the end of each trial.
             Each function must accept two parameters with the following types in this order:
-            :class:`~optuna.study.Study` and :class:`~optuna.FrozenTrial`.
+            :class:`~optuna.study.Study` and :class:`~optuna.trial.FrozenTrial`.
             Please note that this is not a ``callbacks`` argument of `lightgbm.train()`_ .
 
         model_dir:
             A directory to save boosters. By default, it is set to :obj:`None` and no boosters are
             saved. Please set shared directory (e.g., directories on NFS) if you want to access
-            :meth:`~optuna.integration.LightGBMTuner.get_best_booster` in distributed environments.
-            Otherwise, it may raise :obj:`ValueError`. If the directory does not exist, it will be
-            created. The filenames of the boosters will be ``{model_dir}/{trial_number}.pkl``
-            (e.g., ``./boosters/0.pkl``).
+            :meth:`~optuna.integration.lightgbm.LightGBMTuner.get_best_booster` in distributed
+            environments. Otherwise, it may raise :obj:`ValueError`. If the directory does not
+            exist, it will be created. The filenames of the boosters will be
+            ``{model_dir}/{trial_number}.pkl`` (e.g., ``./boosters/0.pkl``).
 
         verbosity:
             A verbosity level to change Optuna's logging level. The level is aligned to
@@ -814,7 +817,7 @@ class LightGBMTuner(_LightGBMBaseTuner):
         categorical_feature: str = "auto",
         early_stopping_rounds: Optional[int] = None,
         evals_result: Optional[Dict[Any, Any]] = None,
-        verbose_eval: Optional[Union[bool, int]] = True,
+        verbose_eval: Optional[Union[bool, int, str]] = "warn",
         learning_rates: Optional[List[float]] = None,
         keep_training_booster: bool = False,
         callbacks: Optional[List[Callable[..., Any]]] = None,
@@ -861,19 +864,6 @@ class LightGBMTuner(_LightGBMBaseTuner):
         if valid_sets is None:
             raise ValueError("`valid_sets` is required.")
 
-    @property  # type: ignore
-    @deprecated(
-        "1.4.0",
-        text=(
-            "Please get the best booster via "
-            ":class:`~optuna.integration.lightgbm.LightGBMTuner.get_best_booster` instead."
-        ),
-    )
-    def best_booster(self) -> "lgb.Booster":
-        """Return the best booster."""
-
-        return self.get_best_booster()
-
     def _create_objective(
         self,
         target_param_names: List[str],
@@ -924,13 +914,13 @@ class LightGBMTunerCV(_LightGBMBaseTuner):
         optuna_callbacks:
             List of Optuna callback functions that are invoked at the end of each trial.
             Each function must accept two parameters with the following types in this order:
-            :class:`~optuna.study.Study` and :class:`~optuna.FrozenTrial`.
+            :class:`~optuna.study.Study` and :class:`~optuna.trial.FrozenTrial`.
             Please note that this is not a ``callbacks`` argument of `lightgbm.train()`_ .
 
         model_dir:
             A directory to save boosters. By default, it is set to :obj:`None` and no boosters are
             saved. Please set shared directory (e.g., directories on NFS) if you want to access
-            :meth:`~optuna.integration.LightGBMTunerCV.get_best_booster`
+            :meth:`~optuna.integration.lightgbm.LightGBMTunerCV.get_best_booster`
             in distributed environments.
             Otherwise, it may raise :obj:`ValueError`. If the directory does not exist, it will be
             created. The filenames of the boosters will be ``{model_dir}/{trial_number}.pkl``
@@ -955,7 +945,7 @@ class LightGBMTunerCV(_LightGBMBaseTuner):
                 Please suppress such messages to show the progress bars properly.
 
         return_cvbooster:
-            Flag to enable :meth:`~optuna.integration.LightGBMTunerCV.get_best_booster`.
+            Flag to enable :meth:`~optuna.integration.lightgbm.LightGBMTunerCV.get_best_booster`.
 
         optuna_seed:
             ``seed`` of :class:`~optuna.samplers.TPESampler` for random number generator
@@ -993,7 +983,7 @@ class LightGBMTunerCV(_LightGBMBaseTuner):
         categorical_feature: str = "auto",
         early_stopping_rounds: Optional[int] = None,
         fpreproc: Optional[Callable[..., Any]] = None,
-        verbose_eval: Optional[Union[bool, int]] = True,
+        verbose_eval: Optional[Union[bool, int]] = None,
         show_stdv: bool = True,
         seed: int = 0,
         callbacks: Optional[List[Callable[..., Any]]] = None,
@@ -1004,7 +994,7 @@ class LightGBMTunerCV(_LightGBMBaseTuner):
         verbosity: Optional[int] = None,
         show_progress_bar: bool = True,
         model_dir: Optional[str] = None,
-        return_cvbooster: Optional[bool] = None,
+        return_cvbooster: bool = False,
         *,
         optuna_seed: Optional[int] = None,
     ) -> None:
@@ -1037,10 +1027,7 @@ class LightGBMTunerCV(_LightGBMBaseTuner):
         self.lgbm_kwargs["show_stdv"] = show_stdv
         self.lgbm_kwargs["seed"] = seed
         self.lgbm_kwargs["fpreproc"] = fpreproc
-        if return_cvbooster is not None:
-            if version.parse(lgb.__version__) < version.parse("3.0.0"):
-                raise ValueError("return_cvbooster requires lightgbm>=3.0.0.")
-            self.lgbm_kwargs["return_cvbooster"] = return_cvbooster
+        self.lgbm_kwargs["return_cvbooster"] = return_cvbooster
 
     def _create_objective(
         self,
