@@ -3,7 +3,7 @@ from typing import Any
 from packaging import version
 
 import optuna
-from optuna._deprecated import deprecated
+from optuna._deprecated import deprecated_class
 from optuna._imports import try_import
 
 
@@ -22,7 +22,7 @@ if not _imports.is_successful():
     TrackerCallback = object  # NOQA
 
 
-@deprecated("2.4.0")
+@deprecated_class("2.4.0", "4.0.0")
 class FastAIV1PruningCallback(TrackerCallback):
     """FastAI callback to prune unpromising trials for fastai.
 
