@@ -2,7 +2,7 @@ import os
 from typing import Dict
 
 import optuna
-from optuna._experimental import experimental
+from optuna._experimental import experimental_class
 from optuna._imports import try_import
 
 
@@ -11,8 +11,8 @@ with try_import() as _imports:
     import tensorflow as tf
 
 
-@experimental("2.0.0")
-class TensorBoardCallback(object):
+@experimental_class("2.0.0")
+class TensorBoardCallback:
     """Callback to track Optuna trials with TensorBoard.
 
     This callback adds relevant information that is tracked by Optuna to TensorBoard.
@@ -56,32 +56,25 @@ class TensorBoardCallback(object):
     def _add_distributions(
         self, distributions: Dict[str, optuna.distributions.BaseDistribution]
     ) -> None:
-        real_distributions = (
-            optuna.distributions.UniformDistribution,
-            optuna.distributions.LogUniformDistribution,
-            optuna.distributions.DiscreteUniformDistribution,
-        )
-        int_distributions = (
-            optuna.distributions.IntUniformDistribution,
-            optuna.distributions.IntLogUniformDistribution,
-        )
-        categorical_distributions = (optuna.distributions.CategoricalDistribution,)
+
         supported_distributions = (
-            real_distributions + int_distributions + categorical_distributions
+            optuna.distributions.CategoricalDistribution,
+            optuna.distributions.FloatDistribution,
+            optuna.distributions.IntDistribution,
         )
 
         for param_name, param_distribution in distributions.items():
-            if isinstance(param_distribution, real_distributions):
+            if isinstance(param_distribution, optuna.distributions.FloatDistribution):
                 self._hp_params[param_name] = hp.HParam(
                     param_name,
                     hp.RealInterval(float(param_distribution.low), float(param_distribution.high)),
                 )
-            elif isinstance(param_distribution, int_distributions):
+            elif isinstance(param_distribution, optuna.distributions.IntDistribution):
                 self._hp_params[param_name] = hp.HParam(
                     param_name,
                     hp.IntInterval(param_distribution.low, param_distribution.high),
                 )
-            elif isinstance(param_distribution, categorical_distributions):
+            elif isinstance(param_distribution, optuna.distributions.CategoricalDistribution):
                 self._hp_params[param_name] = hp.HParam(
                     param_name,
                     hp.Discrete(param_distribution.choices),
