@@ -1,6 +1,7 @@
 import types
 from typing import Any
 from typing import Callable
+from typing import Container
 from typing import Dict
 from typing import Iterable
 from typing import List
@@ -12,7 +13,8 @@ from typing import Union
 
 from optuna import logging
 from optuna import multi_objective
-from optuna._deprecated import deprecated
+from optuna._deprecated import deprecated_class
+from optuna._deprecated import deprecated_func
 from optuna.pruners import NopPruner
 from optuna.storages import BaseStorage
 from optuna.study import create_study as _create_study
@@ -41,7 +43,7 @@ _logger = logging.get_logger(__name__)
 #
 # TODO(ohta): Consider to add `objective_labels` argument.
 # See: https://github.com/optuna/optuna/pull/1054#issuecomment-616382152
-@deprecated("2.4.0", "4.0.0")
+@deprecated_func("2.4.0", "4.0.0")
 def create_study(
     directions: List[str],
     study_name: Optional[str] = None,
@@ -63,7 +65,7 @@ def create_study(
                 x = trial.suggest_float("x", 0, 5)
                 y = trial.suggest_float("y", 0, 3)
 
-                v0 = 4 * x ** 2 + 4 * y ** 2
+                v0 = 4 * x**2 + 4 * y**2
                 v1 = (x - 5) ** 2 + (y - 5) ** 2
                 return v0, v1
 
@@ -128,12 +130,14 @@ def create_study(
         load_if_exists=load_if_exists,
     )
 
-    study.set_system_attr("multi_objective:study:directions", list(directions))
+    study._storage.set_study_system_attr(
+        study._study_id, "multi_objective:study:directions", list(directions)
+    )
 
     return MultiObjectiveStudy(study)
 
 
-@deprecated("2.4.0", "4.0.0")
+@deprecated_func("2.4.0", "4.0.0")
 def load_study(
     study_name: str,
     storage: Union[str, BaseStorage],
@@ -160,7 +164,7 @@ def load_study(
                 x = trial.suggest_float("x", 0, 5)
                 y = trial.suggest_float("y", 0, 3)
 
-                v0 = 4 * x ** 2 + 4 * y ** 2
+                v0 = 4 * x**2 + 4 * y**2
                 v1 = (x - 5) ** 2 + (y - 5) ** 2
                 return v0, v1
 
@@ -205,8 +209,8 @@ def load_study(
     return MultiObjectiveStudy(study)
 
 
-@deprecated("2.4.0", "4.0.0")
-class MultiObjectiveStudy(object):
+@deprecated_class("2.4.0", "4.0.0")
+class MultiObjectiveStudy:
     """A study corresponds to a multi-objective optimization task, i.e., a set of trials.
 
     This object provides interfaces to run a new
@@ -223,7 +227,9 @@ class MultiObjectiveStudy(object):
         self._study = study
 
         self._directions = []
-        for d in study.system_attrs["multi_objective:study:directions"]:
+        for d in study._storage.get_study_system_attrs(study._study_id)[
+            "multi_objective:study:directions"
+        ]:
             if d == "minimize":
                 self._directions.append(StudyDirection.MINIMIZE)
             elif d == "maximize":
@@ -303,7 +309,7 @@ class MultiObjectiveStudy(object):
                     x = trial.suggest_float("x", 0, 5)
                     y = trial.suggest_float("y", 0, 3)
 
-                    v0 = 4 * x ** 2 + 4 * y ** 2
+                    v0 = 4 * x**2 + 4 * y**2
                     v1 = (x - 5) ** 2 + (y - 5) ** 2
                     return v0, v1
 
@@ -359,7 +365,7 @@ class MultiObjectiveStudy(object):
             A dictionary containing all system attributes.
         """
 
-        return self._study.system_attrs
+        return self._study._storage.get_study_system_attrs(self._study._study_id)
 
     def set_user_attr(self, key: str, value: Any) -> None:
         """Set a user attribute to the study.
@@ -384,7 +390,7 @@ class MultiObjectiveStudy(object):
 
         """
 
-        self._study.set_system_attr(key, value)
+        self._study._storage.set_study_system_attr(self._study._study_id, key, value)
 
     def enqueue_trial(self, params: Dict[str, Any]) -> None:
         """Enqueue a trial with given parameter values.
@@ -400,7 +406,7 @@ class MultiObjectiveStudy(object):
                 Parameter values to pass your objective function.
         """
 
-        self._study.enqueue_trial(params)
+        self._study.enqueue_trial(params, skip_if_exists=False)
 
     @property
     def trials(self) -> List["multi_objective.trial.FrozenMultiObjectiveTrial"]:
@@ -419,7 +425,7 @@ class MultiObjectiveStudy(object):
     def get_trials(
         self,
         deepcopy: bool = True,
-        states: Optional[Tuple[TrialState, ...]] = None,
+        states: Optional[Container[TrialState]] = None,
     ) -> List["multi_objective.trial.FrozenMultiObjectiveTrial"]:
         """Return all trials in the study.
 
@@ -480,11 +486,16 @@ class MultiObjectiveStudy(object):
         return self._study._study_id
 
 
-def _log_completed_trial(self: Study, trial: Trial, values: Sequence[float]) -> None:
+def _log_completed_trial(self: Study, trial: FrozenTrial) -> None:
     if not _logger.isEnabledFor(logging.INFO):
         return
 
-    actual_values = multi_objective.trial.MultiObjectiveTrial(trial)._get_values()
+    n_objectives = len(self.directions)
+    frozen_multi_objective_trial = multi_objective.trial.FrozenMultiObjectiveTrial(
+        n_objectives,
+        trial,
+    )
+    actual_values = frozen_multi_objective_trial.values
     _logger.info(
         "Trial {} finished with values: {} with parameters: {}.".format(
             trial.number, actual_values, trial.params
