@@ -3,11 +3,12 @@ from typing import Any
 from typing import Dict
 from typing import List
 from typing import Optional
+from typing import overload
 from typing import Sequence
 from typing import Union
 
 from optuna import multi_objective
-from optuna._deprecated import deprecated
+from optuna._deprecated import deprecated_class
 from optuna.distributions import BaseDistribution
 from optuna.study._study_direction import StudyDirection
 from optuna.trial import FrozenTrial
@@ -18,8 +19,8 @@ from optuna.trial import TrialState
 CategoricalChoiceType = Union[None, bool, int, float, str]
 
 
-@deprecated("2.4.0", "4.0.0")
-class MultiObjectiveTrial(object):
+@deprecated_class("2.4.0", "4.0.0")
+class MultiObjectiveTrial:
     """A trial is a process of evaluating an objective function.
 
     This object is passed to an objective function and provides interfaces to get parameter
@@ -95,6 +96,32 @@ class MultiObjectiveTrial(object):
 
         return self._trial.suggest_int(name, low, high, step=step, log=log)
 
+    @overload
+    def suggest_categorical(self, name: str, choices: Sequence[None]) -> None:
+        ...
+
+    @overload
+    def suggest_categorical(self, name: str, choices: Sequence[bool]) -> bool:
+        ...
+
+    @overload
+    def suggest_categorical(self, name: str, choices: Sequence[int]) -> int:
+        ...
+
+    @overload
+    def suggest_categorical(self, name: str, choices: Sequence[float]) -> float:
+        ...
+
+    @overload
+    def suggest_categorical(self, name: str, choices: Sequence[str]) -> str:
+        ...
+
+    @overload
+    def suggest_categorical(
+        self, name: str, choices: Sequence[CategoricalChoiceType]
+    ) -> CategoricalChoiceType:
+        ...
+
     def suggest_categorical(
         self, name: str, choices: Sequence[CategoricalChoiceType]
     ) -> CategoricalChoiceType:
@@ -169,7 +196,7 @@ class MultiObjectiveTrial(object):
         for further details.
         """
 
-        self._trial.set_system_attr(key, value)
+        self._trial.storage.set_trial_system_attr(self._trial._trial_id, key, value)
 
     @property
     def number(self) -> int:
@@ -235,13 +262,9 @@ class MultiObjectiveTrial(object):
     # This method would be helpful to use the existing pruning
     # integrations for multi-objective optimization.
 
-    def _get_values(self) -> List[Optional[float]]:
-        trial = self._trial.study._storage.get_trial(self._trial._trial_id)
-        return [trial.intermediate_values.get(i) for i in range(self._n_objectives)]
 
-
-@deprecated("2.4.0", "4.0.0")
-class FrozenMultiObjectiveTrial(object):
+@deprecated_class("2.4.0", "4.0.0")
+class FrozenMultiObjectiveTrial:
     """Status and results of a :class:`~optuna.multi_objective.trial.MultiObjectiveTrial`.
 
     Attributes:
