@@ -1,8 +1,7 @@
 import collections
 import itertools
-import random
+from numbers import Real
 from typing import Any
-from typing import cast
 from typing import Dict
 from typing import List
 from typing import Mapping
@@ -11,6 +10,8 @@ from typing import Sequence
 from typing import Union
 import warnings
 
+import numpy as np
+
 from optuna.distributions import BaseDistribution
 from optuna.logging import get_logger
 from optuna.samplers import BaseSampler
@@ -20,7 +21,6 @@ from optuna.trial import TrialState
 
 
 GridValueType = Union[str, float, int, bool, None]
-SortableParamValueSequenceType = Union[List[str], List[float], List[int], List[bool]]
 
 
 _logger = get_logger(__name__)
@@ -42,7 +42,7 @@ class GridSampler(BaseSampler):
             def objective(trial):
                 x = trial.suggest_float("x", -100, 100)
                 y = trial.suggest_int("y", -100, 100)
-                return x ** 2 + y ** 2
+                return x**2 + y**2
 
 
             search_space = {"x": [-50, 0, 50], "y": [-99, 0, 99]}
@@ -70,7 +70,7 @@ class GridSampler(BaseSampler):
             def objective(trial):
                 # The following suggest method specifies integer points between -5 and 5.
                 x = trial.suggest_float("x", -5, 5, step=1)
-                return x ** 2
+                return x**2
 
 
             # Non-int points are specified in the grid.
@@ -85,31 +85,40 @@ class GridSampler(BaseSampler):
         The total number of actual trials may therefore exceed the size of the grid.
 
     Note:
-        The grid is randomly shuffled and the order in which parameter configurations are
-        suggested may vary. This is to reduce duplicate suggestions during distributed
-        optimization.
+        All parameters must be specified when using :class:`~optuna.samplers.GridSampler` with
+        :meth:`~optuna.study.Study.enqueue_trial`.
 
     Args:
         search_space:
             A dictionary whose key and value are a parameter name and the corresponding candidates
             of values, respectively.
+        seed:
+            A seed to fix the order of trials as the grid is randomly shuffled. Please note that
+            it is not recommended using this option in distributed optimization settings since
+            this option cannot ensure the order of trials and may increase the number of duplicate
+            suggestions during distributed optimization.
     """
 
-    def __init__(self, search_space: Mapping[str, Sequence[GridValueType]]) -> None:
+    def __init__(
+        self, search_space: Mapping[str, Sequence[GridValueType]], seed: Optional[int] = None
+    ) -> None:
 
         for param_name, param_values in search_space.items():
             for value in param_values:
                 self._check_value(param_name, value)
 
         self._search_space = collections.OrderedDict()
-        for param_name, param_values in sorted(search_space.items(), key=lambda x: x[0]):
-            param_values = cast(SortableParamValueSequenceType, param_values)
-
-            self._search_space[param_name] = sorted(param_values)
+        for param_name, param_values in sorted(search_space.items()):
+            self._search_space[param_name] = list(param_values)
 
         self._all_grids = list(itertools.product(*self._search_space.values()))
         self._param_names = sorted(search_space.keys())
         self._n_min_trials = len(self._all_grids)
+        self._rng = np.random.RandomState(seed)
+
+    def reseed_rng(self) -> None:
+
+        self._rng.seed()
 
     def infer_relative_search_space(
         self, study: Study, trial: FrozenTrial
@@ -125,6 +134,11 @@ class GridSampler(BaseSampler):
         # object is hard to get at the beginning of trial, while we need the access to the object
         # to validate the sampled value.
 
+        # When the trial is created by RetryFailedTrialCallback or enqueue_trial, we should not
+        # assign a new grid_id.
+        if "grid_id" in trial.system_attrs or "fixed_params" in trial.system_attrs:
+            return {}
+
         target_grids = self._get_unvisited_grid_ids(study)
 
         if len(target_grids) == 0:
@@ -143,7 +157,7 @@ class GridSampler(BaseSampler):
 
         # In distributed optimization, multiple workers may simultaneously pick up the same grid.
         # To make the conflict less frequent, the grid is chosen randomly.
-        grid_id = random.choice(target_grids)
+        grid_id = int(self._rng.choice(target_grids))
 
         study._storage.set_trial_system_attr(trial._trial_id, "search_space", self._search_space)
         study._storage.set_trial_system_attr(trial._trial_id, "grid_id", grid_id)
@@ -158,6 +172,10 @@ class GridSampler(BaseSampler):
         param_distribution: BaseDistribution,
     ) -> Any:
 
+        if "grid_id" not in trial.system_attrs:
+            message = "All parameters must be specified when using GridSampler with enqueue_trial."
+            raise ValueError(message)
+
         if param_name not in self._search_space:
             message = "The parameter name, {}, is not found in the given grid.".format(param_name)
             raise ValueError(message)
@@ -198,11 +216,12 @@ class GridSampler(BaseSampler):
         if param_value is None or isinstance(param_value, (str, int, float, bool)):
             return
 
-        raise ValueError(
+        message = (
             "{} contains a value with the type of {}, which is not supported by "
             "`GridSampler`. Please make sure a value is `str`, `int`, `float`, `bool`"
-            " or `None`.".format(param_name, type(param_value))
+            " or `None` for persistent storage.".format(param_name, type(param_value))
         )
+        warnings.warn(message)
 
     def _get_unvisited_grid_ids(self, study: Study) -> List[int]:
 
@@ -233,6 +252,12 @@ class GridSampler(BaseSampler):
 
         return list(unvisited_grids)
 
+    @staticmethod
+    def _grid_value_equal(value1: GridValueType, value2: GridValueType) -> bool:
+        value1_is_nan = isinstance(value1, Real) and np.isnan(float(value1))
+        value2_is_nan = isinstance(value2, Real) and np.isnan(float(value2))
+        return (value1 == value2) or (value1_is_nan and value2_is_nan)
+
     def _same_search_space(self, search_space: Mapping[str, Sequence[GridValueType]]) -> bool:
 
         if set(search_space.keys()) != set(self._search_space.keys()):
@@ -242,9 +267,8 @@ class GridSampler(BaseSampler):
             if len(search_space[param_name]) != len(self._search_space[param_name]):
                 return False
 
-            param_values = cast(SortableParamValueSequenceType, search_space[param_name])
-            for i, param_value in enumerate(sorted(param_values)):
-                if param_value != self._search_space[param_name][i]:
+            for i, param_value in enumerate(search_space[param_name]):
+                if not self._grid_value_equal(param_value, self._search_space[param_name][i]):
                     return False
 
         return True
