@@ -5,23 +5,19 @@ from typing import Optional
 from typing import Tuple
 
 import numpy as np
-import scipy.special
-from scipy.stats import truncnorm
 
 from optuna import distributions
 from optuna.distributions import BaseDistribution
+from optuna.samplers._tpe import _truncnorm as truncnorm
 
 
 EPS = 1e-12
 SIGMA0_MAGNITUDE = 0.2
 
 _DISTRIBUTION_CLASSES = (
-    distributions.UniformDistribution,
-    distributions.LogUniformDistribution,
-    distributions.DiscreteUniformDistribution,
-    distributions.IntUniformDistribution,
-    distributions.IntLogUniformDistribution,
     distributions.CategoricalDistribution,
+    distributions.FloatDistribution,
+    distributions.IntDistribution,
 )
 
 
@@ -120,20 +116,14 @@ class _ParzenEstimator:
                 # We sample from truncnorm.
                 trunc_low = (low - mus[active]) / sigmas[active]
                 trunc_high = (high - mus[active]) / sigmas[active]
-                samples = np.full((), fill_value=high + 1.0, dtype=np.float64)
-                while (samples >= high).any():
-                    samples = np.where(
-                        samples < high,
-                        samples,
-                        truncnorm.rvs(
-                            trunc_low,
-                            trunc_high,
-                            size=size,
-                            loc=mus[active],
-                            scale=sigmas[active],
-                            random_state=rng,
-                        ),
-                    )
+                samples = truncnorm.rvs(
+                    trunc_low,
+                    trunc_high,
+                    size=size,
+                    loc=mus[active],
+                    scale=sigmas[active],
+                    random_state=rng,
+                )
             samples_dict[param_name] = samples
         samples_dict = self._transform_from_uniform(samples_dict)
         return samples_dict
@@ -169,8 +159,8 @@ class _ParzenEstimator:
                 log_pdf = np.log(categorical_weights.T[samples, :])
             else:
                 # We restore parameters of parzen estimators.
-                low = self._low[param_name]
-                high = self._high[param_name]
+                low = np.asarray(self._low[param_name])
+                high = np.asarray(self._high[param_name])
                 q = self._q[param_name]
                 mus = self._mus[param_name]
                 sigmas = self._sigmas[param_name]
@@ -179,24 +169,29 @@ class _ParzenEstimator:
                 assert mus is not None
                 assert sigmas is not None
 
-                cdf_func = _ParzenEstimator._normal_cdf
-                p_accept = cdf_func(high, mus, sigmas) - cdf_func(low, mus, sigmas)
                 if q is None:
-                    distance = samples[:, None] - mus
-                    mahalanobis = distance / np.maximum(sigmas, EPS)
-                    z = np.sqrt(2 * np.pi) * sigmas
-                    coefficient = 1 / z / p_accept
-                    log_pdf = -0.5 * mahalanobis ** 2 + np.log(coefficient)
+                    log_pdf = truncnorm.logpdf(
+                        samples[:, None],
+                        (low - mus) / sigmas,
+                        (high - mus) / sigmas,
+                        loc=mus,
+                        scale=sigmas,
+                    )
                 else:
                     upper_bound = np.minimum(samples + q / 2.0, high)
                     lower_bound = np.maximum(samples - q / 2.0, low)
-                    cdf = cdf_func(upper_bound[:, None], mus[None], sigmas[None]) - cdf_func(
-                        lower_bound[:, None], mus[None], sigmas[None]
+                    log_gauss_mass = truncnorm._log_gauss_mass(
+                        (lower_bound[:, None] - mus) / sigmas,
+                        (upper_bound[:, None] - mus) / sigmas,
                     )
-                    log_pdf = np.log(cdf + EPS) - np.log(p_accept + EPS)
+                    log_p_accept = truncnorm._log_gauss_mass(
+                        (low - mus) / sigmas, (high - mus) / sigmas
+                    )
+                    log_pdf = log_gauss_mass - log_p_accept
             component_log_pdf += log_pdf
-        ret = scipy.special.logsumexp(component_log_pdf + np.log(self._weights), axis=1)
-        return ret
+        weighted_log_pdf = component_log_pdf + np.log(self._weights)
+        max_ = weighted_log_pdf.max(axis=1)
+        return np.log(np.exp(weighted_log_pdf - max_[:, np.newaxis]).sum(axis=1)) + max_
 
     def _calculate_weights(self, predetermined_weights: Optional[np.ndarray]) -> np.ndarray:
 
@@ -211,6 +206,22 @@ class _ParzenEstimator:
 
         if predetermined_weights is None:
             w = weights_func(n_observations)[:n_observations]
+            if w is not None:
+                if np.any(w < 0):
+                    raise ValueError(
+                        f"The `weights` function is not allowed to return negative values {w}. "
+                        + f"The argument of the `weights` function is {n_observations}."
+                    )
+                if len(w) > 0 and np.sum(w) <= 0:
+                    raise ValueError(
+                        f"The `weight` function is not allowed to return all-zero values {w}."
+                        + f" The argument of the `weights` function is {n_observations}."
+                    )
+                if not np.all(np.isfinite(w)):
+                    raise ValueError(
+                        "The `weights`function is not allowed to return infinite or NaN values "
+                        + f"{w}. The argument of the `weights` function is {n_observations}."
+                    )
         else:
             w = predetermined_weights[:n_observations]
 
@@ -230,34 +241,33 @@ class _ParzenEstimator:
     ) -> Tuple[Optional[float], Optional[float], Optional[float]]:
 
         # We calculate low and high.
-        if isinstance(distribution, distributions.UniformDistribution):
-            low = distribution.low
-            high = distribution.high
-            q = None
-        elif isinstance(distribution, distributions.LogUniformDistribution):
-            low = np.log(distribution.low)
-            high = np.log(distribution.high)
-            q = None
-        elif isinstance(distribution, distributions.DiscreteUniformDistribution):
-            q = distribution.q
-            low = distribution.low - 0.5 * q
-            high = distribution.high + 0.5 * q
-        elif isinstance(distribution, distributions.IntUniformDistribution):
-            q = distribution.step
-            low = distribution.low - 0.5 * q
-            high = distribution.high + 0.5 * q
-        elif isinstance(distribution, distributions.IntLogUniformDistribution):
-            low = np.log(distribution.low - 0.5)
-            high = np.log(distribution.high + 0.5)
-            q = None
+        if isinstance(distribution, distributions.FloatDistribution):
+            if distribution.log:
+                low = np.log(distribution.low)
+                high = np.log(distribution.high)
+                q = None
+            elif distribution.step is not None:
+                q = distribution.step
+                low = distribution.low - 0.5 * q
+                high = distribution.high + 0.5 * q
+            else:
+                low = distribution.low
+                high = distribution.high
+                q = None
+        elif isinstance(distribution, distributions.IntDistribution):
+            if distribution.log:
+                low = np.log(distribution.low - 0.5)
+                high = np.log(distribution.high + 0.5)
+                q = None
+            else:
+                q = distribution.step
+                low = distribution.low - 0.5 * q
+                high = distribution.high + 0.5 * q
         else:
             distribution_list = [
-                distributions.UniformDistribution.__name__,
-                distributions.LogUniformDistribution.__name__,
-                distributions.DiscreteUniformDistribution.__name__,
-                distributions.IntUniformDistribution.__name__,
-                distributions.IntLogUniformDistribution.__name__,
                 distributions.CategoricalDistribution.__name__,
+                distributions.FloatDistribution.__name__,
+                distributions.IntDistribution.__name__,
             ]
             raise NotImplementedError(
                 "The distribution {} is not implemented. "
@@ -279,9 +289,10 @@ class _ParzenEstimator:
             assert isinstance(distribution, _DISTRIBUTION_CLASSES)
             if isinstance(
                 distribution,
-                (distributions.LogUniformDistribution, distributions.IntLogUniformDistribution),
+                (distributions.FloatDistribution, distributions.IntDistribution),
             ):
-                samples = np.log(samples)
+                if distribution.log:
+                    samples = np.log(samples)
 
             transformed[param_name] = samples
         return transformed
@@ -295,29 +306,31 @@ class _ParzenEstimator:
             distribution = self._search_space[param_name]
 
             assert isinstance(distribution, _DISTRIBUTION_CLASSES)
-            if isinstance(distribution, distributions.UniformDistribution):
-                transformed[param_name] = samples
-            elif isinstance(distribution, distributions.LogUniformDistribution):
-                transformed[param_name] = np.exp(samples)
-            elif isinstance(distribution, distributions.DiscreteUniformDistribution):
-                q = self._q[param_name]
-                assert q is not None
-                samples = np.round((samples - distribution.low) / q) * q + distribution.low
-                transformed[param_name] = np.asarray(
-                    np.clip(samples, distribution.low, distribution.high)
-                )
-            elif isinstance(distribution, distributions.IntUniformDistribution):
-                q = self._q[param_name]
-                assert q is not None
-                samples = np.round((samples - distribution.low) / q) * q + distribution.low
-                transformed[param_name] = np.asarray(
-                    np.clip(samples, distribution.low, distribution.high)
-                )
-            elif isinstance(distribution, distributions.IntLogUniformDistribution):
-                samples = np.round(np.exp(samples))
-                transformed[param_name] = np.asarray(
-                    np.clip(samples, distribution.low, distribution.high)
-                )
+            if isinstance(distribution, distributions.FloatDistribution):
+                if distribution.log:
+                    transformed[param_name] = np.exp(samples)
+                elif distribution.step is not None:
+                    q = self._q[param_name]
+                    assert q is not None
+                    samples = np.round((samples - distribution.low) / q) * q + distribution.low
+                    transformed[param_name] = np.asarray(
+                        np.clip(samples, distribution.low, distribution.high)
+                    )
+                else:
+                    transformed[param_name] = samples
+            elif isinstance(distribution, distributions.IntDistribution):
+                if distribution.log:
+                    samples = np.round(np.exp(samples))
+                    transformed[param_name] = np.asarray(
+                        np.clip(samples, distribution.low, distribution.high)
+                    )
+                else:
+                    q = self._q[param_name]
+                    assert q is not None
+                    samples = np.round((samples - distribution.low) / q) * q + distribution.low
+                    transformed[param_name] = np.asarray(
+                        np.clip(samples, distribution.low, distribution.high)
+                    )
             elif isinstance(distribution, distributions.CategoricalDistribution):
                 transformed[param_name] = samples
 
@@ -434,15 +447,6 @@ class _ParzenEstimator:
 
         return mus, sigmas
 
-    @staticmethod
-    def _normal_cdf(x: float, mu: np.ndarray, sigma: np.ndarray) -> np.ndarray:
-
-        mu, sigma = map(np.asarray, (mu, sigma))
-        denominator = x - mu
-        numerator = np.maximum(np.sqrt(2) * sigma, EPS)
-        z = denominator / numerator
-        return 0.5 * (1 + scipy.special.erf(z))
-
     @staticmethod
     def _sample_from_categorical_dist(
         rng: np.random.RandomState, probabilities: np.ndarray
