@@ -2,7 +2,7 @@ import copy
 from datetime import datetime
 import threading
 from typing import Any
-from typing import cast
+from typing import Container
 from typing import Dict
 from typing import Iterator
 from typing import List
@@ -17,8 +17,8 @@ from optuna import distributions  # NOQA
 from optuna.exceptions import DuplicatedStudyError
 from optuna.storages import BaseStorage
 from optuna.storages._base import DEFAULT_STUDY_NAME_PREFIX
+from optuna.study._frozen import FrozenStudy
 from optuna.study._study_direction import StudyDirection
-from optuna.study._study_summary import StudySummary
 from optuna.trial import FrozenTrial
 from optuna.trial import TrialState
 
@@ -51,7 +51,9 @@ class InMemoryStorage(BaseStorage):
         self.__dict__.update(state)
         self._lock = threading.RLock()
 
-    def create_new_study(self, study_name: Optional[str] = None) -> int:
+    def create_new_study(
+        self, directions: Sequence[StudyDirection], study_name: Optional[str] = None
+    ) -> int:
 
         with self._lock:
             study_id = self._max_study_id + 1
@@ -63,7 +65,8 @@ class InMemoryStorage(BaseStorage):
             else:
                 study_uuid = str(uuid.uuid4())
                 study_name = DEFAULT_STUDY_NAME_PREFIX + study_uuid
-            self._studies[study_id] = _StudyInfo(study_name)
+
+            self._studies[study_id] = _StudyInfo(study_name, list(directions))
             self._study_name_to_id[study_name] = study_id
 
             _logger.info("A new study created in memory with name: {}".format(study_name))
@@ -81,22 +84,6 @@ class InMemoryStorage(BaseStorage):
             del self._study_name_to_id[study_name]
             del self._studies[study_id]
 
-    def set_study_directions(self, study_id: int, directions: Sequence[StudyDirection]) -> None:
-
-        with self._lock:
-            self._check_study_id(study_id)
-
-            study = self._studies[study_id]
-            if study.directions[0] != StudyDirection.NOT_SET and study.directions != list(
-                directions
-            ):
-                raise ValueError(
-                    "Cannot overwrite study direction from {} to {}.".format(
-                        study.directions, directions
-                    )
-                )
-            study.directions = list(directions)
-
     def set_study_user_attr(self, study_id: int, key: str, value: Any) -> None:
 
         with self._lock:
@@ -119,13 +106,6 @@ class InMemoryStorage(BaseStorage):
 
             return self._study_name_to_id[study_name]
 
-    def get_study_id_from_trial_id(self, trial_id: int) -> int:
-
-        with self._lock:
-            self._check_trial_id(trial_id)
-
-            return self._trial_id_to_study_id_and_number[trial_id][0]
-
     def get_study_name_from_id(self, study_id: int) -> str:
 
         with self._lock:
@@ -150,33 +130,18 @@ class InMemoryStorage(BaseStorage):
             self._check_study_id(study_id)
             return self._studies[study_id].system_attrs
 
-    def get_all_study_summaries(self) -> List[StudySummary]:
-
+    def get_all_studies(self) -> List[FrozenStudy]:
         with self._lock:
-            return [self._build_study_summary(study_id) for study_id in self._studies]
+            return [self._build_frozen_study(study_id) for study_id in self._studies]
 
-    def _build_study_summary(self, study_id: int) -> StudySummary:
+    def _build_frozen_study(self, study_id: int) -> FrozenStudy:
         study = self._studies[study_id]
-        return StudySummary(
+        return FrozenStudy(
             study_name=study.name,
             direction=None,
             directions=study.directions,
-            best_trial=copy.deepcopy(self._get_trial(study.best_trial_id))
-            if study.best_trial_id is not None
-            else None,
             user_attrs=copy.deepcopy(study.user_attrs),
             system_attrs=copy.deepcopy(study.system_attrs),
-            n_trials=len(study.trials),
-            datetime_start=min(
-                [
-                    cast(datetime, trial.datetime_start)
-                    for trial in self.get_all_trials(study_id, deepcopy=False)
-                    if trial.datetime_start is not None
-                ],
-                default=None,
-            )
-            if study.trials
-            else None,
             study_id=study_id,
         )
 
@@ -216,33 +181,6 @@ class InMemoryStorage(BaseStorage):
             datetime_complete=None,
         )
 
-    def set_trial_state(self, trial_id: int, state: TrialState) -> bool:
-
-        with self._lock:
-            trial = self._get_trial(trial_id)
-            self.check_trial_is_updatable(trial_id, trial.state)
-
-            trial = copy.copy(trial)
-            self.check_trial_is_updatable(trial_id, trial.state)
-
-            if state == TrialState.RUNNING and trial.state != TrialState.WAITING:
-                return False
-
-            trial.state = state
-
-            if state == TrialState.RUNNING:
-                trial.datetime_start = datetime.now()
-
-            if state.is_finished():
-                trial.datetime_complete = datetime.now()
-                self._set_trial(trial_id, trial)
-                study_id = self._trial_id_to_study_id_and_number[trial_id][0]
-                self._update_cache(trial_id, study_id)
-            else:
-                self._set_trial(trial_id, trial)
-
-            return True
-
     def set_trial_param(
         self,
         trial_id: int,
@@ -307,10 +245,11 @@ class InMemoryStorage(BaseStorage):
             self._check_study_id(study_id)
 
             best_trial_id = self._studies[study_id].best_trial_id
+
             if best_trial_id is None:
                 raise ValueError("No trials are completed yet.")
             elif len(self._studies[study_id].directions) > 1:
-                raise ValueError(
+                raise RuntimeError(
                     "Best trial can be obtained only for single-objective optimization."
                 )
             return self.get_trial(best_trial_id)
@@ -323,17 +262,33 @@ class InMemoryStorage(BaseStorage):
             distribution = trial.distributions[param_name]
             return distribution.to_internal_repr(trial.params[param_name])
 
-    def set_trial_values(self, trial_id: int, values: Sequence[float]) -> None:
+    def set_trial_state_values(
+        self, trial_id: int, state: TrialState, values: Optional[Sequence[float]] = None
+    ) -> bool:
 
         with self._lock:
-            trial = self._get_trial(trial_id)
+            trial = copy.copy(self._get_trial(trial_id))
             self.check_trial_is_updatable(trial_id, trial.state)
 
-            trial = copy.copy(trial)
-            self.check_trial_is_updatable(trial_id, trial.state)
+            if state == TrialState.RUNNING and trial.state != TrialState.WAITING:
+                return False
 
-            trial.values = values
-            self._set_trial(trial_id, trial)
+            trial.state = state
+            if values is not None:
+                trial.values = values
+
+            if state == TrialState.RUNNING:
+                trial.datetime_start = datetime.now()
+
+            if state.is_finished():
+                trial.datetime_complete = datetime.now()
+                self._set_trial(trial_id, trial)
+                study_id = self._trial_id_to_study_id_and_number[trial_id][0]
+                self._update_cache(trial_id, study_id)
+            else:
+                self._set_trial(trial_id, trial)
+
+            return True
 
     def _update_cache(self, trial_id: int, study_id: int) -> None:
 
@@ -389,8 +344,6 @@ class InMemoryStorage(BaseStorage):
             trial = self._get_trial(trial_id)
             self.check_trial_is_updatable(trial_id, trial.state)
 
-            self.check_trial_is_updatable(trial_id, trial.state)
-
             trial = copy.copy(trial)
             trial.user_attrs = copy.copy(trial.user_attrs)
             trial.user_attrs[key] = value
@@ -402,8 +355,6 @@ class InMemoryStorage(BaseStorage):
             trial = self._get_trial(trial_id)
             self.check_trial_is_updatable(trial_id, trial.state)
 
-            self.check_trial_is_updatable(trial_id, trial.state)
-
             trial = copy.copy(trial)
             trial.system_attrs = copy.copy(trial.system_attrs)
             trial.system_attrs[key] = value
@@ -428,7 +379,7 @@ class InMemoryStorage(BaseStorage):
         self,
         study_id: int,
         deepcopy: bool = True,
-        states: Optional[Tuple[TrialState, ...]] = None,
+        states: Optional[Container[TrialState]] = None,
     ) -> List[FrozenTrial]:
 
         with self._lock:
@@ -448,9 +399,6 @@ class InMemoryStorage(BaseStorage):
 
         return trials
 
-    def read_trials_from_remote_storage(self, study_id: int) -> None:
-        self._check_study_id(study_id)
-
     def _check_study_id(self, study_id: int) -> None:
 
         if study_id not in self._studies:
@@ -463,11 +411,11 @@ class InMemoryStorage(BaseStorage):
 
 
 class _StudyInfo:
-    def __init__(self, name: str) -> None:
+    def __init__(self, name: str, directions: List[StudyDirection]) -> None:
         self.trials: List[FrozenTrial] = []
         self.param_distribution: Dict[str, distributions.BaseDistribution] = {}
         self.user_attrs: Dict[str, Any] = {}
         self.system_attrs: Dict[str, Any] = {}
         self.name: str = name
-        self.directions: List[StudyDirection] = [StudyDirection.NOT_SET]
+        self.directions: List[StudyDirection] = directions
         self.best_trial_id: Optional[int] = None
