@@ -1,6 +1,9 @@
+import enum
+import math
 from typing import Any
 from typing import List
 from typing import Optional
+from typing import Tuple
 
 from sqlalchemy import asc
 from sqlalchemy import CheckConstraint
@@ -16,13 +19,19 @@ from sqlalchemy import orm
 from sqlalchemy import String
 from sqlalchemy import Text
 from sqlalchemy import UniqueConstraint
-from sqlalchemy.ext.declarative import declarative_base
 
 from optuna import distributions
 from optuna.study._study_direction import StudyDirection
 from optuna.trial import TrialState
 
 
+try:
+    from sqlalchemy.orm import declarative_base
+except ImportError:
+    # TODO(c-bata): Remove this after dropping support for SQLAlchemy v1.3 or prior.
+    from sqlalchemy.ext.declarative import declarative_base
+
+
 # Don't modify this version number anymore.
 # The schema management functionality has been moved to alembic.
 SCHEMA_VERSION = 12
@@ -32,6 +41,8 @@ MAX_VERSION_LENGTH = 256
 
 NOT_FOUND_MSG = "Record does not exist."
 
+FLOAT_PRECISION = 53
+
 BaseModel: Any = declarative_base()
 
 
@@ -85,19 +96,6 @@ class StudyDirectionModel(BaseModel):
         StudyModel, backref=orm.backref("directions", cascade="all, delete-orphan")
     )
 
-    @classmethod
-    def find_by_study_and_objective(
-        cls, study: StudyModel, objective: int, session: orm.Session
-    ) -> Optional["StudyDirectionModel"]:
-        study_direction = (
-            session.query(cls)
-            .filter(cls.study_id == study.study_id)
-            .filter(cls.objective == objective)
-            .one_or_none()
-        )
-
-        return study_direction
-
     @classmethod
     def where_study_id(cls, study_id: int, session: orm.Session) -> List["StudyDirectionModel"]:
 
@@ -342,7 +340,7 @@ class TrialParamModel(BaseModel):
     param_id = Column(Integer, primary_key=True)
     trial_id = Column(Integer, ForeignKey("trials.trial_id"))
     param_name = Column(String(MAX_INDEXED_STRING_LENGTH))
-    param_value = Column(Float)
+    param_value = Column(Float(precision=FLOAT_PRECISION))
     distribution_json = Column(Text())
 
     trial = orm.relationship(
@@ -408,17 +406,48 @@ class TrialParamModel(BaseModel):
 
 
 class TrialValueModel(BaseModel):
+    class TrialValueType(enum.Enum):
+        FINITE = 1
+        INF_POS = 2
+        INF_NEG = 3
+
     __tablename__ = "trial_values"
     __table_args__: Any = (UniqueConstraint("trial_id", "objective"),)
     trial_value_id = Column(Integer, primary_key=True)
     trial_id = Column(Integer, ForeignKey("trials.trial_id"), nullable=False)
     objective = Column(Integer, nullable=False)
-    value = Column(Float, nullable=False)
+    value = Column(Float(precision=FLOAT_PRECISION), nullable=True)
+    value_type = Column(Enum(TrialValueType), nullable=False)
 
     trial = orm.relationship(
         TrialModel, backref=orm.backref("values", cascade="all, delete-orphan")
     )
 
+    @classmethod
+    def value_to_stored_repr(
+        cls,
+        value: float,
+    ) -> Tuple[Optional[float], TrialValueType]:
+        if value == float("inf"):
+            return (None, cls.TrialValueType.INF_POS)
+        elif value == float("-inf"):
+            return (None, cls.TrialValueType.INF_NEG)
+        else:
+            return (value, cls.TrialValueType.FINITE)
+
+    @classmethod
+    def stored_repr_to_value(cls, value: Optional[float], float_type: TrialValueType) -> float:
+        if float_type == cls.TrialValueType.INF_POS:
+            assert value is None
+            return float("inf")
+        elif float_type == cls.TrialValueType.INF_NEG:
+            assert value is None
+            return float("-inf")
+        else:
+            assert float_type == cls.TrialValueType.FINITE
+            assert value is not None
+            return value
+
     @classmethod
     def find_by_trial_and_objective(
         cls, trial: TrialModel, objective: int, session: orm.Session
@@ -444,17 +473,56 @@ class TrialValueModel(BaseModel):
 
 
 class TrialIntermediateValueModel(BaseModel):
+    class TrialIntermediateValueType(enum.Enum):
+        FINITE = 1
+        INF_POS = 2
+        INF_NEG = 3
+        NAN = 4
+
     __tablename__ = "trial_intermediate_values"
     __table_args__: Any = (UniqueConstraint("trial_id", "step"),)
     trial_intermediate_value_id = Column(Integer, primary_key=True)
     trial_id = Column(Integer, ForeignKey("trials.trial_id"), nullable=False)
     step = Column(Integer, nullable=False)
-    intermediate_value = Column(Float, nullable=False)
+    intermediate_value = Column(Float(precision=FLOAT_PRECISION), nullable=True)
+    intermediate_value_type = Column(Enum(TrialIntermediateValueType), nullable=False)
 
     trial = orm.relationship(
         TrialModel, backref=orm.backref("intermediate_values", cascade="all, delete-orphan")
     )
 
+    @classmethod
+    def intermediate_value_to_stored_repr(
+        cls,
+        value: float,
+    ) -> Tuple[Optional[float], TrialIntermediateValueType]:
+        if math.isnan(value):
+            return (None, cls.TrialIntermediateValueType.NAN)
+        elif value == float("inf"):
+            return (None, cls.TrialIntermediateValueType.INF_POS)
+        elif value == float("-inf"):
+            return (None, cls.TrialIntermediateValueType.INF_NEG)
+        else:
+            return (value, cls.TrialIntermediateValueType.FINITE)
+
+    @classmethod
+    def stored_repr_to_intermediate_value(
+        cls, value: Optional[float], float_type: TrialIntermediateValueType
+    ) -> float:
+        if float_type == cls.TrialIntermediateValueType.NAN:
+            assert value is None
+            return float("nan")
+        elif float_type == cls.TrialIntermediateValueType.INF_POS:
+            assert value is None
+            return float("inf")
+        elif float_type == cls.TrialIntermediateValueType.INF_NEG:
+            assert value is None
+            return float("-inf")
+        else:
+            assert float_type == cls.TrialIntermediateValueType.FINITE
+            assert value is not None
+            return value
+
     @classmethod
     def find_by_trial_and_step(
         cls, trial: TrialModel, step: int, session: orm.Session
