@@ -3,34 +3,40 @@ from typing import Any
 from typing import Dict
 from typing import List
 from typing import Optional
+from typing import overload
 from typing import Sequence
-from typing import Union
+import warnings
 
 from optuna import distributions
 from optuna import logging
-from optuna._experimental import experimental
+from optuna._deprecated import deprecated_func
+from optuna.distributions import _convert_old_distribution_to_new_distribution
 from optuna.distributions import BaseDistribution
+from optuna.distributions import CategoricalChoiceType
 from optuna.distributions import CategoricalDistribution
-from optuna.distributions import DiscreteUniformDistribution
-from optuna.distributions import IntLogUniformDistribution
-from optuna.distributions import IntUniformDistribution
-from optuna.distributions import LogUniformDistribution
-from optuna.distributions import UniformDistribution
+from optuna.distributions import FloatDistribution
+from optuna.distributions import IntDistribution
 from optuna.trial._base import BaseTrial
 from optuna.trial._state import TrialState
 
 
 _logger = logging.get_logger(__name__)
-
-CategoricalChoiceType = Union[None, bool, int, float, str]
+_suggest_deprecated_msg = "Use :func:`~optuna.trial.FrozenTrial.suggest_float` instead."
 
 
 class FrozenTrial(BaseTrial):
     """Status and results of a :class:`~optuna.trial.Trial`.
 
-    This object has the same methods as :class:`~optuna.trial.Trial`, and it suggests best
-    parameter values among performed trials. In contrast to :class:`~optuna.trial.Trial`,
-    :class:`~optuna.trial.FrozenTrial` does not depend on :class:`~optuna.study.Study`, and it is
+    An object of this class has the same methods as :class:`~optuna.trial.Trial`, but is not
+    associated with, nor has any references to a :class:`~optuna.study.Study`.
+
+    It is therefore not possible to make persistent changes to a storage from this object by
+    itself, for instance by using :func:`~optuna.trial.FrozenTrial.set_user_attr`.
+
+    It will suggest the parameter values stored in :attr:`params` and will not sample values from
+    any distributions.
+
+    It can be passed to objective functions (see :func:`~optuna.study.Study.optimize`) and is
     useful for deploying optimization results.
 
     Example:
@@ -43,8 +49,8 @@ class FrozenTrial(BaseTrial):
 
 
             def objective(trial):
-                x = trial.suggest_uniform("x", -1, 1)
-                return x ** 2
+                x = trial.suggest_float("x", -1, 1)
+                return x**2
 
 
             study = optuna.create_study()
@@ -71,12 +77,12 @@ class FrozenTrial(BaseTrial):
 
 
                 def objective(trial):
-                    x = trial.suggest_uniform("x", -1, 1)
+                    x = trial.suggest_float("x", -1, 1)
 
                     # this user attribute always differs
                     trial.set_user_attr("evaluation time", datetime.datetime.now())
 
-                    return x ** 2
+                    return x**2
 
 
                 study = optuna.create_study()
@@ -103,24 +109,27 @@ class FrozenTrial(BaseTrial):
             :class:`TrialState` of the :class:`~optuna.trial.Trial`.
         value:
             Objective value of the :class:`~optuna.trial.Trial`.
+            ``value`` and ``values`` must not be specified at the same time.
         values:
             Sequence of objective values of the :class:`~optuna.trial.Trial`.
             The length is greater than 1 if the problem is multi-objective optimization.
+            ``value`` and ``values`` must not be specified at the same time.
         datetime_start:
             Datetime where the :class:`~optuna.trial.Trial` started.
         datetime_complete:
             Datetime where the :class:`~optuna.trial.Trial` finished.
         params:
             Dictionary that contains suggested parameters.
+        distributions:
+            Dictionary that contains the distributions of :attr:`params`.
         user_attrs:
             Dictionary that contains the attributes of the :class:`~optuna.trial.Trial` set with
             :func:`optuna.trial.Trial.set_user_attr`.
+        system_attrs:
+            Dictionary that contains the attributes of the :class:`~optuna.trial.Trial` set with
+            :func:`optuna.trial.Trial.set_system_attr`.
         intermediate_values:
             Intermediate objective values set with :func:`optuna.trial.Trial.report`.
-
-    Raises:
-        :exc:`ValueError`:
-            If both ``value`` and ``values`` are specified.
     """
 
     def __init__(
@@ -158,22 +167,6 @@ class FrozenTrial(BaseTrial):
         self._distributions = distributions
         self._trial_id = trial_id
 
-    # Ordered list of fields required for `__repr__`, `__hash__` and dataframe creation.
-    # TODO(hvy): Remove this list in Python 3.7 as the order of `self.__dict__` is preserved.
-    _ordered_fields = [
-        "number",
-        "_values",
-        "datetime_start",
-        "datetime_complete",
-        "params",
-        "_distributions",
-        "user_attrs",
-        "system_attrs",
-        "intermediate_values",
-        "_trial_id",
-        "state",
-    ]
-
     def __eq__(self, other: Any) -> bool:
 
         if not isinstance(other, FrozenTrial):
@@ -196,7 +189,7 @@ class FrozenTrial(BaseTrial):
 
     def __hash__(self) -> int:
 
-        return hash(tuple(getattr(self, field) for field in self._ordered_fields))
+        return hash(tuple(getattr(self, field) for field in self.__dict__))
 
     def __repr__(self) -> str:
 
@@ -207,7 +200,7 @@ class FrozenTrial(BaseTrial):
                     field=field if not field.startswith("_") else field[1:],
                     value=repr(getattr(self, field)),
                 )
-                for field in self._ordered_fields
+                for field in self.__dict__
             )
             + ", value=None",
         )
@@ -222,48 +215,51 @@ class FrozenTrial(BaseTrial):
         log: bool = False,
     ) -> float:
 
-        if step is not None:
-            if log:
-                raise ValueError("The parameter `step` is not supported when `log` is True.")
-            else:
-                return self._suggest(name, DiscreteUniformDistribution(low=low, high=high, q=step))
-        else:
-            if log:
-                return self._suggest(name, LogUniformDistribution(low=low, high=high))
-            else:
-                return self._suggest(name, UniformDistribution(low=low, high=high))
+        return self._suggest(name, FloatDistribution(low, high, log=log, step=step))
 
+    @deprecated_func("3.0.0", "6.0.0", text=_suggest_deprecated_msg)
     def suggest_uniform(self, name: str, low: float, high: float) -> float:
 
-        return self._suggest(name, UniformDistribution(low=low, high=high))
+        return self.suggest_float(name, low, high)
 
+    @deprecated_func("3.0.0", "6.0.0", text=_suggest_deprecated_msg)
     def suggest_loguniform(self, name: str, low: float, high: float) -> float:
 
-        return self._suggest(name, LogUniformDistribution(low=low, high=high))
+        return self.suggest_float(name, low, high, log=True)
 
+    @deprecated_func("3.0.0", "6.0.0", text=_suggest_deprecated_msg)
     def suggest_discrete_uniform(self, name: str, low: float, high: float, q: float) -> float:
 
-        discrete = DiscreteUniformDistribution(low=low, high=high, q=q)
-        return self._suggest(name, discrete)
+        return self.suggest_float(name, low, high, step=q)
 
     def suggest_int(self, name: str, low: int, high: int, step: int = 1, log: bool = False) -> int:
+        return int(self._suggest(name, IntDistribution(low, high, log=log, step=step)))
 
-        if step != 1:
-            if log:
-                raise ValueError(
-                    "The parameter `step != 1` is not supported when `log` is True."
-                    "The specified `step` is {}.".format(step)
-                )
-            else:
-                distribution: Union[
-                    IntUniformDistribution, IntLogUniformDistribution
-                ] = IntUniformDistribution(low=low, high=high, step=step)
-        else:
-            if log:
-                distribution = IntLogUniformDistribution(low=low, high=high)
-            else:
-                distribution = IntUniformDistribution(low=low, high=high, step=step)
-        return int(self._suggest(name, distribution))
+    @overload
+    def suggest_categorical(self, name: str, choices: Sequence[None]) -> None:
+        ...
+
+    @overload
+    def suggest_categorical(self, name: str, choices: Sequence[bool]) -> bool:
+        ...
+
+    @overload
+    def suggest_categorical(self, name: str, choices: Sequence[int]) -> int:
+        ...
+
+    @overload
+    def suggest_categorical(self, name: str, choices: Sequence[float]) -> float:
+        ...
+
+    @overload
+    def suggest_categorical(self, name: str, choices: Sequence[str]) -> str:
+        ...
+
+    @overload
+    def suggest_categorical(
+        self, name: str, choices: Sequence[CategoricalChoiceType]
+    ) -> CategoricalChoiceType:
+        ...
 
     def suggest_categorical(
         self, name: str, choices: Sequence[CategoricalChoiceType]
@@ -310,6 +306,7 @@ class FrozenTrial(BaseTrial):
 
         self._user_attrs[key] = value
 
+    @deprecated_func("3.1.0", "6.0.0")
     def set_system_attr(self, key: str, value: Any) -> None:
 
         self._system_attrs[key] = value
@@ -361,7 +358,7 @@ class FrozenTrial(BaseTrial):
         value = self._params[name]
         param_value_in_internal_repr = distribution.to_internal_repr(value)
         if not distribution._contains(param_value_in_internal_repr):
-            raise ValueError(
+            warnings.warn(
                 "The value {} of the parameter '{}' is out of "
                 "the range of the distribution {}.".format(value, name, distribution)
             )
@@ -446,7 +443,6 @@ class FrozenTrial(BaseTrial):
 
     @property
     def distributions(self) -> Dict[str, BaseDistribution]:
-        """Dictionary that contains the distributions of :attr:`params`."""
 
         return self._distributions
 
@@ -477,7 +473,7 @@ class FrozenTrial(BaseTrial):
 
     @property
     def last_step(self) -> Optional[int]:
-        """Return the maximum step of `intermediate_values` in the trial.
+        """Return the maximum step of :attr:`intermediate_values` in the trial.
 
         Returns:
             The maximum step of intermediates.
@@ -502,7 +498,6 @@ class FrozenTrial(BaseTrial):
             return None
 
 
-@experimental("2.0.0")
 def create_trial(
     *,
     state: TrialState = TrialState.COMPLETE,
@@ -522,12 +517,12 @@ def create_trial(
 
             import optuna
             from optuna.distributions import CategoricalDistribution
-            from optuna.distributions import UniformDistribution
+            from optuna.distributions import FloatDistribution
 
             trial = optuna.trial.create_trial(
                 params={"x": 1.0, "y": 0},
                 distributions={
-                    "x": UniformDistribution(0, 10),
+                    "x": FloatDistribution(0, 10),
                     "y": CategoricalDistribution([-1, 0, 1]),
                 },
                 value=5.0,
@@ -560,10 +555,12 @@ def create_trial(
             Trial state.
         value:
             Trial objective value. Must be specified if ``state`` is :class:`TrialState.COMPLETE`.
+            ``value`` and ``values`` must not be specified at the same time.
         values:
             Sequence of the trial objective values. The length is greater than 1 if the problem is
             multi-objective optimization.
             Must be specified if ``state`` is :class:`TrialState.COMPLETE`.
+            ``value`` and ``values`` must not be specified at the same time.
         params:
             Dictionary with suggested parameters of the trial.
         distributions:
@@ -577,14 +574,14 @@ def create_trial(
 
     Returns:
         Created trial.
-
-    Raises:
-        :exc:`ValueError`:
-            If both ``value`` and ``values`` are specified.
     """
 
     params = params or {}
     distributions = distributions or {}
+    distributions = {
+        key: _convert_old_distribution_to_new_distribution(dist)
+        for key, dist in distributions.items()
+    }
     user_attrs = user_attrs or {}
     system_attrs = system_attrs or {}
     intermediate_values = intermediate_values or {}
