@@ -1,6 +1,7 @@
 from collections import OrderedDict
 from typing import Callable
 from typing import List
+from typing import NamedTuple
 from typing import Optional
 
 import optuna
@@ -12,6 +13,7 @@ from optuna.trial import FrozenTrial
 from optuna.trial import TrialState
 from optuna.visualization._plotly_imports import _imports
 from optuna.visualization._utils import _check_plot_args
+from optuna.visualization._utils import _filter_nonfinite
 
 
 if _imports.is_successful():
@@ -23,6 +25,52 @@ if _imports.is_successful():
 logger = get_logger(__name__)
 
 
+class _ImportancesInfo(NamedTuple):
+    importance_values: List[float]
+    param_names: List[str]
+    importance_labels: List[str]
+    target_name: str
+
+
+def _get_importances_info(
+    study: Study,
+    evaluator: Optional[BaseImportanceEvaluator],
+    params: Optional[List[str]],
+    target: Optional[Callable[[FrozenTrial], float]],
+    target_name: str,
+) -> _ImportancesInfo:
+    _check_plot_args(study, target, target_name)
+
+    trials = _filter_nonfinite(
+        study.get_trials(deepcopy=False, states=(TrialState.COMPLETE,)), target=target
+    )
+
+    if len(trials) == 0:
+        logger.warning("Study instance does not contain completed trials.")
+        return _ImportancesInfo(
+            importance_values=[],
+            param_names=[],
+            importance_labels=[],
+            target_name=target_name,
+        )
+
+    importances = optuna.importance.get_param_importances(
+        study, evaluator=evaluator, params=params, target=target
+    )
+
+    importances = OrderedDict(reversed(list(importances.items())))
+    importance_values = list(importances.values())
+    param_names = list(importances.keys())
+    importance_labels = [f"{val:.2f}" if val >= 0.01 else "<0.01" for val in importance_values]
+
+    return _ImportancesInfo(
+        importance_values=importance_values,
+        param_names=param_names,
+        importance_labels=importance_labels,
+        target_name=target_name,
+    )
+
+
 def plot_param_importances(
     study: Study,
     evaluator: Optional[BaseImportanceEvaluator] = None,
@@ -85,51 +133,39 @@ def plot_param_importances(
 
     Returns:
         A :class:`plotly.graph_objs.Figure` object.
-
-    Raises:
-        :exc:`ValueError`:
-            If ``target`` is :obj:`None` and ``study`` is being used for multi-objective
-            optimization.
     """
 
     _imports.check()
-    _check_plot_args(study, target, target_name)
+
+    importances_info = _get_importances_info(study, evaluator, params, target, target_name)
+    hover_template = _get_hover_template(importances_info, study)
+    return _get_importances_plot(importances_info, hover_template)
+
+
+def _get_importances_plot(info: _ImportancesInfo, hover_template: List[str]) -> "go.Figure":
 
     layout = go.Layout(
         title="Hyperparameter Importances",
-        xaxis={"title": f"Importance for {target_name}"},
+        xaxis={"title": f"Importance for {info.target_name}"},
         yaxis={"title": "Hyperparameter"},
         showlegend=False,
     )
 
-    # Importances cannot be evaluated without completed trials.
-    # Return an empty figure for consistency with other visualization functions.
-    trials = [trial for trial in study.trials if trial.state == TrialState.COMPLETE]
-    if len(trials) == 0:
-        logger.warning("Study instance does not contain completed trials.")
+    param_names = info.param_names
+    importance_values = info.importance_values
+
+    if len(importance_values) == 0:
         return go.Figure(data=[], layout=layout)
 
-    importances = optuna.importance.get_param_importances(
-        study, evaluator=evaluator, params=params, target=target
-    )
-
-    importances = OrderedDict(reversed(list(importances.items())))
-    importance_values = list(importances.values())
-    param_names = list(importances.keys())
-
     fig = go.Figure(
         data=[
             go.Bar(
                 x=importance_values,
                 y=param_names,
-                text=importance_values,
-                texttemplate="%{text:.2f}",
+                text=info.importance_labels,
                 textposition="outside",
                 cliponaxis=False,  # Ensure text is not clipped.
-                hovertemplate=[
-                    _make_hovertext(param_name, importance, study)
-                    for param_name, importance in importances.items()
-                ],
+                hovertemplate=hover_template,
                 marker_color=plotly.colors.sequential.Blues[-4],
                 orientation="h",
             )
@@ -151,3 +187,12 @@ def _make_hovertext(param_name: str, importance: float, study: Study) -> str:
     return "{} ({}): {}<extra></extra>".format(
         param_name, _get_distribution(param_name, study).__class__.__name__, importance
     )
+
+
+def _get_hover_template(importances_info: _ImportancesInfo, study: Study) -> List[str]:
+    return [
+        _make_hovertext(param_name, importance, study)
+        for param_name, importance in zip(
+            importances_info.param_names, importances_info.importance_values
+        )
+    ]
