@@ -3,13 +3,13 @@ from packaging import version
 from optuna._imports import try_import
 
 
-with try_import() as _imports:  # NOQA
-    import plotly  # NOQA
+with try_import() as _imports:
+    import plotly
     from plotly import __version__ as plotly_version
-    import plotly.graph_objs as go  # NOQA
-    from plotly.graph_objs import Contour  # NOQA
-    from plotly.graph_objs import Scatter  # NOQA
-    from plotly.subplots import make_subplots  # NOQA
+    import plotly.graph_objs as go
+    from plotly.graph_objs import Contour
+    from plotly.graph_objs import Scatter
+    from plotly.subplots import make_subplots
 
     if version.parse(plotly_version) < version.parse("4.0.0"):
         raise ImportError(
@@ -19,3 +19,5 @@ with try_import() as _imports:  # NOQA
             "For further information, please refer to the installation guide of plotly. ",
             name="plotly",
         )
+
+__all__ = ["_imports", "plotly", "go", "Contour", "Scatter", "make_subplots"]
