@@ -6,22 +6,30 @@ from typing import Type
 from typing import Union
 from unittest import mock
 
-import _jsonnet
-import allennlp.data
-import allennlp.data.dataset_readers
-import allennlp.data.tokenizers
-import allennlp.models
-import allennlp.modules
-import allennlp.modules.seq2vec_encoders
-import allennlp.modules.text_field_embedders
-import allennlp.training
 import pytest
-import torch.optim
 
 import optuna
+from optuna._imports import try_import
 from optuna.integration.allennlp import AllenNLPPruningCallback
 from optuna.integration.allennlp._pruner import _create_pruner
-from optuna.testing.integration import DeterministicPruner
+from optuna.integration.allennlp._variables import _VariableManager
+from optuna.testing.pruners import DeterministicPruner
+
+
+with try_import():
+    import _jsonnet
+    import allennlp.data
+    import allennlp.data.dataset_readers
+    import allennlp.data.tokenizers
+    import allennlp.models
+    import allennlp.modules
+    import allennlp.modules.seq2vec_encoders
+    import allennlp.modules.text_field_embedders
+    import allennlp.training
+    import psutil
+    import torch.optim
+
+pytestmark = pytest.mark.integration
 
 
 def test_build_params() -> None:
@@ -79,7 +87,7 @@ def test_build_params_when_optuna_and_environment_variable_both_exist() -> None:
     os.environ.pop("LEARNING_RATE")
     os.environ.pop("DROPOUT")
 
-    # Optuna trial overwrites a parameter specified by environment variable
+    # Optuna trial overwrites a parameter specified by environment variable.
     assert params["trainer"]["optimizer"]["lr"] == 1e-2
     path = params["model"]["text_field_embedder"]["token_embedders"]["token_characters"]["dropout"]
     assert path == 0.0
@@ -195,11 +203,12 @@ def test_allennlp_executor_with_options() -> None:
         with open(os.path.join(executor._serialization_dir, "metrics.json"), "w") as fout:
             json.dump({executor._metrics: 1.0}, fout)
 
+        expected_include_packages = [package_name, "optuna.integration.allennlp"]
         with mock.patch("allennlp.commands.train.train_model", return_value=None) as mock_obj:
             executor.run()
             assert mock_obj.call_args[1]["force"]
             assert mock_obj.call_args[1]["file_friendly_logging"]
-            assert mock_obj.call_args[1]["include_package"] == [package_name]
+            assert mock_obj.call_args[1]["include_package"] == expected_include_packages
 
 
 def test_dump_best_config() -> None:
@@ -265,24 +274,27 @@ def test_allennlp_pruning_callback() -> None:
     with tempfile.TemporaryDirectory() as tmp_dir:
 
         def objective(trial: optuna.Trial) -> float:
-            reader = allennlp.data.dataset_readers.TextClassificationJsonReader(
-                tokenizer=allennlp.data.tokenizers.WhitespaceTokenizer(),
+            reader = allennlp.data.dataset_readers.TextClassificationJsonReader(  # type: ignore
+                tokenizer=allennlp.data.tokenizers.WhitespaceTokenizer(),  # type: ignore
             )
-            data_loader = allennlp.data.data_loaders.MultiProcessDataLoader(
+            data_loader = allennlp.data.data_loaders.MultiProcessDataLoader(  # type: ignore
                 reader=reader,
                 data_path="tests/integration_tests/allennlp_tests/pruning_test.jsonl",
                 batch_size=16,
             )
-            vocab = allennlp.data.Vocabulary.from_instances(data_loader.iter_instances())
+            vocab = allennlp.data.Vocabulary.from_instances(  # type: ignore
+                data_loader.iter_instances()
+            )
+
             data_loader.index_with(vocab)
 
-            embedder = allennlp.modules.text_field_embedders.BasicTextFieldEmbedder(
-                {"tokens": allennlp.modules.Embedding(50, vocab=vocab)}
+            embedder = allennlp.modules.text_field_embedders.BasicTextFieldEmbedder(  # type: ignore # NOQA
+                {"tokens": allennlp.modules.Embedding(50, vocab=vocab)}  # type: ignore
             )
-            encoder = allennlp.modules.seq2vec_encoders.GruSeq2VecEncoder(
+            encoder = allennlp.modules.seq2vec_encoders.GruSeq2VecEncoder(  # type: ignore
                 input_size=50, hidden_size=50
             )
-            model = allennlp.models.BasicClassifier(
+            model = allennlp.models.BasicClassifier(  # type: ignore
                 text_field_embedder=embedder, seq2vec_encoder=encoder, vocab=vocab
             )
             optimizer = torch.optim.SGD(model.parameters(), lr=0.1)
@@ -359,13 +371,18 @@ def test_allennlp_pruning_callback_with_invalid_storage() -> None:
         ),
     ],
 )
+@pytest.mark.parametrize(
+    "input_config_file",
+    [
+        "tests/integration_tests/allennlp_tests/example_with_executor_and_pruner.jsonnet",
+        "tests/integration_tests/allennlp_tests/example_with_executor_and_pruner_distributed.jsonnet",  # noqa: E501
+    ],
+)
 def test_allennlp_pruning_callback_with_executor(
-    pruner_class: Type[optuna.pruners.BasePruner], pruner_kwargs: Dict[str, Union[int, float]]
+    pruner_class: Type[optuna.pruners.BasePruner],
+    pruner_kwargs: Dict[str, Union[int, float]],
+    input_config_file: str,
 ) -> None:
-    input_config_file = (
-        "tests/integration_tests/allennlp_tests/example_with_executor_and_pruner.jsonnet"
-    )
-
     def run_allennlp_executor(pruner: optuna.pruners.BasePruner) -> None:
         study = optuna.create_study(direction="maximize", pruner=pruner, storage=storage)
         trial = optuna.trial.Trial(study, study._storage.create_new_trial(study._study_id))
@@ -379,9 +396,14 @@ def test_allennlp_pruning_callback_with_executor(
         storage = "sqlite:///" + os.path.join(tmp_dir, pruner_name, "result.db")
         serialization_dir = os.path.join(tmp_dir, pruner_name, "allennlp")
 
-        pruner = pruner_class(**pruner_kwargs)  # type: ignore
+        pruner = pruner_class(**pruner_kwargs)
         run_allennlp_executor(pruner)
-        ret_pruner = _create_pruner()
+        process = psutil.Process()
+        manager = _VariableManager(process.ppid())
+        ret_pruner = _create_pruner(
+            manager.get_value("pruner_class"),
+            manager.get_value("pruner_kwargs"),
+        )
 
         assert isinstance(ret_pruner, pruner_class)
         for key, value in pruner_kwargs.items():
