@@ -4,19 +4,28 @@ import typing
 from unittest.mock import Mock
 from unittest.mock import patch
 
-import chainer
-import chainer.links as L
-from chainer.training import triggers
 import numpy as np
 import pytest
 
 import optuna
+from optuna._imports import try_import
 from optuna.integration.chainer import ChainerPruningExtension
-from optuna.testing.integration import create_running_trial
-from optuna.testing.integration import DeterministicPruner
+from optuna.testing.pruners import DeterministicPruner
 
 
-class FixedValueDataset(chainer.dataset.DatasetMixin):
+with try_import() as _imports:
+    import chainer
+    from chainer.dataset import DatasetMixin  # type: ignore[attr-defined]
+    import chainer.links as L
+    from chainer.training import triggers
+
+if not _imports.is_successful():
+    DatasetMixin = object  # NOQA
+
+pytestmark = pytest.mark.integration
+
+
+class FixedValueDataset(DatasetMixin):
 
     size = 16
 
@@ -32,22 +41,35 @@ class FixedValueDataset(chainer.dataset.DatasetMixin):
 def test_chainer_pruning_extension_trigger() -> None:
 
     study = optuna.create_study()
-    trial = create_running_trial(study, 1.0)
+    trial = study.ask()
 
     extension = ChainerPruningExtension(trial, "main/loss", (1, "epoch"))
-    assert isinstance(extension._pruner_trigger, triggers.IntervalTrigger)
-    extension = ChainerPruningExtension(trial, "main/loss", triggers.IntervalTrigger(1, "epoch"))
-    assert isinstance(extension._pruner_trigger, triggers.IntervalTrigger)
+    assert isinstance(
+        extension._pruner_trigger, triggers.IntervalTrigger  # type: ignore[attr-defined]
+    )
     extension = ChainerPruningExtension(
-        trial, "main/loss", triggers.ManualScheduleTrigger(1, "epoch")
+        trial, "main/loss", triggers.IntervalTrigger(1, "epoch")  # type: ignore[attr-defined]
+    )
+    assert isinstance(
+        extension._pruner_trigger, triggers.IntervalTrigger  # type: ignore[attr-defined]
+    )
+    extension = ChainerPruningExtension(
+        trial,
+        "main/loss",
+        triggers.ManualScheduleTrigger(1, "epoch"),  # type: ignore[attr-defined]
+    )
+    assert isinstance(
+        extension._pruner_trigger, triggers.ManualScheduleTrigger  # type: ignore[attr-defined]
     )
-    assert isinstance(extension._pruner_trigger, triggers.ManualScheduleTrigger)
 
     with pytest.raises(TypeError):
-        ChainerPruningExtension(trial, "main/loss", triggers.TimeTrigger(1.0))  # type: ignore
+        ChainerPruningExtension(
+            trial, "main/loss", triggers.TimeTrigger(1.0)  # type: ignore[attr-defined]
+        )
 
 
 def test_chainer_pruning_extension() -> None:
+    @typing.no_type_check
     def objective(trial: optuna.trial.Trial) -> float:
 
         model = L.Classifier(chainer.Sequential(L.Linear(None, 2)))
@@ -77,7 +99,7 @@ def test_chainer_pruning_extension() -> None:
 def test_chainer_pruning_extension_observation_nan() -> None:
 
     study = optuna.create_study(pruner=DeterministicPruner(True))
-    trial = create_running_trial(study, 1.0)
+    trial = study.ask()
     extension = ChainerPruningExtension(trial, "main/loss", (1, "epoch"))
 
     MockTrainer = namedtuple("MockTrainer", ("observation", "updater"))
@@ -86,38 +108,44 @@ def test_chainer_pruning_extension_observation_nan() -> None:
 
     with patch.object(extension, "_observation_exists", Mock(return_value=True)) as mock:
         with pytest.raises(optuna.TrialPruned):
-            extension(trainer)  # type: ignore
+            extension(trainer)
         assert mock.call_count == 1
 
 
 def test_observation_exists() -> None:
 
     study = optuna.create_study()
-    trial = create_running_trial(study, 1.0)
+    trial = study.ask()
     MockTrainer = namedtuple("MockTrainer", ("observation",))
     trainer = MockTrainer(observation={"OK": 0})
 
     # Trigger is deactivated. Return False whether trainer has observation or not.
-    with patch.object(triggers.IntervalTrigger, "__call__", Mock(return_value=False)) as mock:
+    with patch.object(
+        triggers.IntervalTrigger,  # type: ignore[attr-defined]
+        "__call__",
+        Mock(return_value=False),
+    ) as mock:
         extension = ChainerPruningExtension(trial, "NG", (1, "epoch"))
-        assert extension._observation_exists(trainer) is False  # type: ignore
+        assert extension._observation_exists(trainer) is False
         extension = ChainerPruningExtension(trial, "OK", (1, "epoch"))
-        assert extension._observation_exists(trainer) is False  # type: ignore
+        assert extension._observation_exists(trainer) is False
         assert mock.call_count == 2
 
     # Trigger is activated. Return True if trainer has observation.
-    with patch.object(triggers.IntervalTrigger, "__call__", Mock(return_value=True)) as mock:
+    with patch.object(
+        triggers.IntervalTrigger, "__call__", Mock(return_value=True)  # type: ignore[attr-defined]
+    ) as mock:
         extension = ChainerPruningExtension(trial, "NG", (1, "epoch"))
-        assert extension._observation_exists(trainer) is False  # type: ignore
+        assert extension._observation_exists(trainer) is False
         extension = ChainerPruningExtension(trial, "OK", (1, "epoch"))
-        assert extension._observation_exists(trainer) is True  # type: ignore
+        assert extension._observation_exists(trainer) is True
         assert mock.call_count == 2
 
 
 def test_get_float_value() -> None:
 
     assert 1.0 == ChainerPruningExtension._get_float_value(1.0)
-    assert 1.0 == ChainerPruningExtension._get_float_value(chainer.Variable(np.array([1.0])))
+    assert 1.0 == ChainerPruningExtension._get_float_value(
+        chainer.Variable(np.array([1.0]))  # type: ignore[attr-defined]
+    )
     assert math.isnan(ChainerPruningExtension._get_float_value(float("nan")))
-    with pytest.raises(TypeError):
-        ChainerPruningExtension._get_float_value([])  # type: ignore
