@@ -17,14 +17,11 @@ from optuna import TrialPruned
 from optuna.integration.chainermn import ChainerMNStudy
 from optuna.integration.chainermn import ChainerMNTrial
 from optuna.pruners import BasePruner
-from optuna.samplers import BaseSampler
 from optuna.storages import BaseStorage
 from optuna.storages import InMemoryStorage
 from optuna.storages import RDBStorage
-from optuna.testing.integration import DeterministicPruner
-from optuna.testing.sampler import DeterministicRelativeSampler
-from optuna.testing.storage import StorageSupplier
-from optuna.trial import Trial
+from optuna.testing.pruners import DeterministicPruner
+from optuna.testing.storages import StorageSupplier
 from optuna.trial import TrialState
 
 
@@ -39,13 +36,15 @@ except ImportError:
 STORAGE_MODES = ["sqlite"]
 PRUNER_INIT_FUNCS = [lambda: pruners.MedianPruner(), lambda: pruners.SuccessiveHalvingPruner()]
 
+pytestmark = pytest.mark.integration
 
-class Func(object):
+
+class Func:
     def __init__(self) -> None:
 
         self.suggested_values: Dict[int, Dict[str, Any]] = {}
 
-    def __call__(self, trial: ChainerMNTrial, comm: CommunicatorBase) -> float:
+    def __call__(self, trial: ChainerMNTrial, comm: "CommunicatorBase") -> float:
 
         x = trial.suggest_float("x", -10, 10)
         y = trial.suggest_float("y", 20, 30, log=True)
@@ -60,7 +59,7 @@ class Func(object):
 
 
 class MultiNodeStorageSupplier(StorageSupplier):
-    def __init__(self, storage_specifier: str, comm: CommunicatorBase) -> None:
+    def __init__(self, storage_specifier: str, comm: "CommunicatorBase") -> None:
 
         super().__init__(storage_specifier)
         self.comm = comm
@@ -93,7 +92,7 @@ class MultiNodeStorageSupplier(StorageSupplier):
 
 
 @pytest.fixture
-def comm() -> CommunicatorBase:
+def comm() -> "CommunicatorBase":
 
     if not _available:
         pytest.skip("This test requires ChainerMN.")
@@ -101,10 +100,10 @@ def comm() -> CommunicatorBase:
     return chainermn.create_communicator("naive")
 
 
-class TestChainerMNStudy(object):
+class TestChainerMNStudy:
     @staticmethod
     @pytest.mark.parametrize("storage_mode", STORAGE_MODES)
-    def test_init(storage_mode: str, comm: CommunicatorBase) -> None:
+    def test_init(storage_mode: str, comm: "CommunicatorBase") -> None:
 
         with MultiNodeStorageSupplier(storage_mode, comm) as storage:
             study = TestChainerMNStudy._create_shared_study(storage, comm)
@@ -114,29 +113,29 @@ class TestChainerMNStudy(object):
 
     @staticmethod
     @pytest.mark.parametrize("storage_mode", STORAGE_MODES)
-    def test_init_with_multiple_study_names(storage_mode: str, comm: CommunicatorBase) -> None:
+    def test_init_with_multiple_study_names(storage_mode: str, comm: "CommunicatorBase") -> None:
 
         TestChainerMNStudy._check_multi_node(comm)
 
         with MultiNodeStorageSupplier(storage_mode, comm) as storage:
             # Create study_name for each rank.
-            name = create_study(storage).study_name
+            name = create_study(storage=storage).study_name
             study = Study(name, storage)
 
             with pytest.raises(ValueError):
                 ChainerMNStudy(study, comm)
 
     @staticmethod
-    def test_init_with_incompatible_storage(comm: CommunicatorBase) -> None:
+    def test_init_with_incompatible_storage(comm: "CommunicatorBase") -> None:
 
-        study = create_study(InMemoryStorage(), study_name="in-memory-study")
+        study = create_study(storage=InMemoryStorage(), study_name="in-memory-study")
 
         with pytest.raises(ValueError):
             ChainerMNStudy(study, comm)
 
     @staticmethod
     @pytest.mark.parametrize("storage_mode", STORAGE_MODES)
-    def test_optimize(storage_mode: str, comm: CommunicatorBase) -> None:
+    def test_optimize(storage_mode: str, comm: "CommunicatorBase") -> None:
 
         with MultiNodeStorageSupplier(storage_mode, comm) as storage:
             study = TestChainerMNStudy._create_shared_study(storage, comm)
@@ -158,7 +157,7 @@ class TestChainerMNStudy(object):
     @pytest.mark.parametrize("storage_mode", STORAGE_MODES)
     @pytest.mark.parametrize("pruner_init_func", PRUNER_INIT_FUNCS)
     def test_pruning(
-        storage_mode: str, pruner_init_func: Callable[[], BasePruner], comm: CommunicatorBase
+        storage_mode: str, pruner_init_func: Callable[[], BasePruner], comm: "CommunicatorBase"
     ) -> None:
 
         with MultiNodeStorageSupplier(storage_mode, comm) as storage:
@@ -183,7 +182,7 @@ class TestChainerMNStudy(object):
 
     @staticmethod
     @pytest.mark.parametrize("storage_mode", STORAGE_MODES)
-    def test_failure(storage_mode: str, comm: CommunicatorBase) -> None:
+    def test_failure(storage_mode: str, comm: "CommunicatorBase") -> None:
 
         with MultiNodeStorageSupplier(storage_mode, comm) as storage:
             study = TestChainerMNStudy._create_shared_study(storage, comm)
@@ -218,60 +217,29 @@ class TestChainerMNStudy(object):
             failed_trials = [t for t in mn_study.trials if t.state == TrialState.FAIL]
             assert len(failed_trials) == n_trials + 1
 
-    @staticmethod
-    @pytest.mark.parametrize("storage_mode", STORAGE_MODES)
-    def test_relative_sampling(storage_mode: str, comm: CommunicatorBase) -> None:
-
-        relative_search_space = {
-            "x": distributions.UniformDistribution(low=-10, high=10),
-            "y": distributions.LogUniformDistribution(low=20, high=30),
-            "z": distributions.CategoricalDistribution(choices=(-1.0, 1.0)),
-        }
-        relative_params = {"x": 1.0, "y": 25.0, "z": -1.0}
-        sampler = DeterministicRelativeSampler(
-            relative_search_space, relative_params  # type: ignore
-        )
-
-        with MultiNodeStorageSupplier(storage_mode, comm) as storage:
-            study = TestChainerMNStudy._create_shared_study(storage, comm, sampler=sampler)
-            mn_study = ChainerMNStudy(study, comm)
-
-            # Invoke optimize.
-            n_trials = 20
-            func = Func()
-            mn_study.optimize(func, n_trials=n_trials)
-
-            # Assert trial counts.
-            assert len(mn_study.trials) == n_trials
-
-            # Assert the parameters in `relative_params` have been suggested among all nodes.
-            for trial in mn_study.trials:
-                assert trial.params == relative_params
-
     @staticmethod
     def _create_shared_study(
         storage: BaseStorage,
-        comm: CommunicatorBase,
+        comm: "CommunicatorBase",
         pruner: Optional[BasePruner] = None,
-        sampler: Optional[BaseSampler] = None,
     ) -> Study:
 
-        name_local = create_study(storage).study_name if comm.rank == 0 else None
+        name_local = create_study(storage=storage).study_name if comm.rank == 0 else None
         name_bcast = comm.mpi_comm.bcast(name_local)
 
-        return Study(name_bcast, storage, pruner=pruner, sampler=sampler)
+        return Study(name_bcast, storage, pruner=pruner)
 
     @staticmethod
-    def _check_multi_node(comm: CommunicatorBase) -> None:
+    def _check_multi_node(comm: "CommunicatorBase") -> None:
 
         if comm.size < 2:
             pytest.skip("This test is for multi-node only.")
 
 
-class TestChainerMNTrial(object):
+class TestChainerMNTrial:
     @staticmethod
     @pytest.mark.parametrize("storage_mode", STORAGE_MODES)
-    def test_init(storage_mode: str, comm: CommunicatorBase) -> None:
+    def test_init(storage_mode: str, comm: "CommunicatorBase") -> None:
 
         with MultiNodeStorageSupplier(storage_mode, comm) as storage:
             study = TestChainerMNStudy._create_shared_study(storage, comm)
@@ -282,7 +250,7 @@ class TestChainerMNTrial(object):
 
     @staticmethod
     @pytest.mark.parametrize("storage_mode", STORAGE_MODES)
-    def test_suggest_float(storage_mode: str, comm: CommunicatorBase) -> None:
+    def test_suggest_float(storage_mode: str, comm: "CommunicatorBase") -> None:
 
         with MultiNodeStorageSupplier(storage_mode, comm) as storage:
             study = TestChainerMNStudy._create_shared_study(storage, comm)
@@ -299,7 +267,7 @@ class TestChainerMNTrial(object):
                 assert x1 == x2
 
                 with pytest.raises(ValueError):
-                    mn_trial.suggest_loguniform("x1", low1, high1)
+                    mn_trial.suggest_float("x1", low1, high1, log=True)
 
             low2 = 1e-7
             high2 = 1e-2
@@ -309,8 +277,7 @@ class TestChainerMNTrial(object):
                 x3 = mn_trial.suggest_float("x2", low2, high2, log=True)
                 assert low2 <= x3 <= high2
 
-                x4 = mn_trial.suggest_loguniform("x2", low2, high2)
-
+                x4 = mn_trial.suggest_float("x2", low2, high2, log=True)
                 assert x3 == x4
 
                 with pytest.raises(ValueError):
@@ -318,70 +285,33 @@ class TestChainerMNTrial(object):
 
     @staticmethod
     @pytest.mark.parametrize("storage_mode", STORAGE_MODES)
-    def test_suggest_uniform(storage_mode: str, comm: CommunicatorBase) -> None:
-
-        with MultiNodeStorageSupplier(storage_mode, comm) as storage:
-            study = TestChainerMNStudy._create_shared_study(storage, comm)
-            low = 0.5
-            high = 1.0
-            for _ in range(10):
-                mn_trial = _create_new_chainermn_trial(study, comm)
-
-                x1 = mn_trial.suggest_float("x", low, high)
-                assert low <= x1 <= high
-
-                x2 = mn_trial.suggest_float("x", low, high)
-                assert x1 == x2
-
-                with pytest.raises(ValueError):
-                    mn_trial.suggest_loguniform("x", low, high)
-
-    @staticmethod
-    @pytest.mark.parametrize("storage_mode", STORAGE_MODES)
-    def test_suggest_loguniform(storage_mode: str, comm: CommunicatorBase) -> None:
-
-        with MultiNodeStorageSupplier(storage_mode, comm) as storage:
-            study = TestChainerMNStudy._create_shared_study(storage, comm)
-            low = 1e-7
-            high = 1e-2
-            for _ in range(10):
-                mn_trial = _create_new_chainermn_trial(study, comm)
-
-                x1 = mn_trial.suggest_loguniform("x", low, high)
-                assert low <= x1 <= high
-
-                x2 = mn_trial.suggest_loguniform("x", low, high)
-                assert x1 == x2
-
-                with pytest.raises(ValueError):
-                    mn_trial.suggest_float("x", low, high)
-
-    @staticmethod
-    @pytest.mark.parametrize("storage_mode", STORAGE_MODES)
-    def test_suggest_discrete_uniform(storage_mode: str, comm: CommunicatorBase) -> None:
+    def test_suggest_float_with_step(storage_mode: str, comm: "CommunicatorBase") -> None:
 
         with MultiNodeStorageSupplier(storage_mode, comm) as storage:
             study = TestChainerMNStudy._create_shared_study(storage, comm)
             low = 0.0
             high = 10.0
-            q = 1.0
+            step = 1.0
             for _ in range(10):
                 mn_trial = _create_new_chainermn_trial(study, comm)
 
-                x1 = mn_trial.suggest_discrete_uniform("x", low, high, q)
+                x1 = mn_trial.suggest_float("x", low, high, step=step)
                 assert low <= x1 <= high
 
-                x2 = mn_trial.suggest_discrete_uniform("x", low, high, q)
+                x2 = mn_trial.suggest_float("x", low, high, step=step)
                 assert x1 == x2
 
-                with pytest.raises(ValueError):
+                if comm.rank == 0:
+                    with pytest.warns(RuntimeWarning):
+                        mn_trial.suggest_float("x", low, high)
+                else:
                     mn_trial.suggest_float("x", low, high)
 
     @staticmethod
     @pytest.mark.parametrize("storage_mode", STORAGE_MODES)
     @pytest.mark.parametrize("enable_log", [False, True])
     def test_suggest_int_step1(
-        storage_mode: str, comm: CommunicatorBase, enable_log: bool
+        storage_mode: str, comm: "CommunicatorBase", enable_log: bool
     ) -> None:
 
         with MultiNodeStorageSupplier(storage_mode, comm) as storage:
@@ -403,12 +333,12 @@ class TestChainerMNTrial(object):
 
     @staticmethod
     @pytest.mark.parametrize("storage_mode", STORAGE_MODES)
-    def test_suggest_int_step2(storage_mode: str, comm: CommunicatorBase) -> None:
+    def test_suggest_int_step2(storage_mode: str, comm: "CommunicatorBase") -> None:
 
         with MultiNodeStorageSupplier(storage_mode, comm) as storage:
             study = TestChainerMNStudy._create_shared_study(storage, comm)
             low = 1
-            high = 10
+            high = 9
             step = 2
             for _ in range(10):
                 mn_trial = _create_new_chainermn_trial(study, comm)
@@ -424,7 +354,7 @@ class TestChainerMNTrial(object):
 
     @staticmethod
     @pytest.mark.parametrize("storage_mode", STORAGE_MODES)
-    def test_suggest_categorical(storage_mode: str, comm: CommunicatorBase) -> None:
+    def test_suggest_categorical(storage_mode: str, comm: "CommunicatorBase") -> None:
 
         with MultiNodeStorageSupplier(storage_mode, comm) as storage:
             study = TestChainerMNStudy._create_shared_study(storage, comm)
@@ -445,7 +375,7 @@ class TestChainerMNTrial(object):
     @pytest.mark.parametrize("storage_mode", STORAGE_MODES)
     @pytest.mark.parametrize("is_pruning", [True, False])
     def test_report_and_should_prune(
-        storage_mode: str, comm: CommunicatorBase, is_pruning: bool
+        storage_mode: str, comm: "CommunicatorBase", is_pruning: bool
     ) -> None:
 
         with MultiNodeStorageSupplier(storage_mode, comm) as storage:
@@ -458,7 +388,7 @@ class TestChainerMNTrial(object):
 
     @staticmethod
     @pytest.mark.parametrize("storage_mode", STORAGE_MODES)
-    def test_params(storage_mode: str, comm: CommunicatorBase) -> None:
+    def test_params(storage_mode: str, comm: "CommunicatorBase") -> None:
 
         with MultiNodeStorageSupplier(storage_mode, comm) as storage:
             study = TestChainerMNStudy._create_shared_study(storage, comm)
@@ -469,7 +399,7 @@ class TestChainerMNTrial(object):
 
     @staticmethod
     @pytest.mark.parametrize("storage_mode", STORAGE_MODES)
-    def test_distributions(storage_mode: str, comm: CommunicatorBase) -> None:
+    def test_distributions(storage_mode: str, comm: "CommunicatorBase") -> None:
 
         with MultiNodeStorageSupplier(storage_mode, comm) as storage:
             study = TestChainerMNStudy._create_shared_study(storage, comm)
@@ -482,7 +412,7 @@ class TestChainerMNTrial(object):
 
     @staticmethod
     @pytest.mark.parametrize("storage_mode", STORAGE_MODES)
-    def test_user_attrs(storage_mode: str, comm: CommunicatorBase) -> None:
+    def test_user_attrs(storage_mode: str, comm: "CommunicatorBase") -> None:
 
         with MultiNodeStorageSupplier(storage_mode, comm) as storage:
             study = TestChainerMNStudy._create_shared_study(storage, comm)
@@ -493,18 +423,7 @@ class TestChainerMNTrial(object):
 
     @staticmethod
     @pytest.mark.parametrize("storage_mode", STORAGE_MODES)
-    def test_system_attrs(storage_mode: str, comm: CommunicatorBase) -> None:
-
-        with MultiNodeStorageSupplier(storage_mode, comm) as storage:
-            study = TestChainerMNStudy._create_shared_study(storage, comm)
-            mn_trial = _create_new_chainermn_trial(study, comm)
-
-            mn_trial.set_system_attr("system_message", "test")
-            assert mn_trial.system_attrs["system_message"] == "test"
-
-    @staticmethod
-    @pytest.mark.parametrize("storage_mode", STORAGE_MODES)
-    def test_call_with_mpi(storage_mode: str, comm: CommunicatorBase) -> None:
+    def test_call_with_mpi(storage_mode: str, comm: "CommunicatorBase") -> None:
 
         with MultiNodeStorageSupplier(storage_mode, comm) as storage:
             study = TestChainerMNStudy._create_shared_study(storage, comm)
@@ -519,7 +438,7 @@ class TestChainerMNTrial(object):
 
     @staticmethod
     @pytest.mark.parametrize("storage_mode", STORAGE_MODES)
-    def test_datetime_start(storage_mode: str, comm: CommunicatorBase) -> None:
+    def test_datetime_start(storage_mode: str, comm: "CommunicatorBase") -> None:
 
         with MultiNodeStorageSupplier(storage_mode, comm) as storage:
             study = TestChainerMNStudy._create_shared_study(storage, comm)
@@ -529,12 +448,11 @@ class TestChainerMNTrial(object):
 
 
 def _create_new_chainermn_trial(
-    study: Study, comm: CommunicatorBase
+    study: Study, comm: "CommunicatorBase"
 ) -> integration.chainermn.ChainerMNTrial:
 
     if comm.rank == 0:
-        trial_id = study._storage.create_new_trial(study._study_id)
-        trial = Trial(study, trial_id)
+        trial = study.ask()
         mn_trial = integration.chainermn.ChainerMNTrial(trial, comm)
     else:
         mn_trial = integration.chainermn.ChainerMNTrial(None, comm)
