@@ -1,12 +1,17 @@
-from keras import Sequential
-from keras.layers import Dense
 import numpy as np
 import pytest
 
 import optuna
+from optuna._imports import try_import
 from optuna.integration import KerasPruningCallback
-from optuna.testing.integration import create_running_trial
-from optuna.testing.integration import DeterministicPruner
+from optuna.testing.pruners import DeterministicPruner
+
+
+with try_import():
+    from keras import Sequential
+    from keras.layers import Dense
+
+pytestmark = pytest.mark.integration
 
 
 @pytest.mark.parametrize("interval, epochs", [(1, 1), (2, 1), (2, 2)])
@@ -43,7 +48,7 @@ def test_keras_pruning_callback(interval: int, epochs: int) -> None:
 def test_keras_pruning_callback_observation_isnan() -> None:
 
     study = optuna.create_study(pruner=DeterministicPruner(True))
-    trial = create_running_trial(study, 1.0)
+    trial = study.ask()
     callback = KerasPruningCallback(trial, "loss")
 
     with pytest.raises(optuna.TrialPruned):
@@ -56,7 +61,7 @@ def test_keras_pruning_callback_observation_isnan() -> None:
 def test_keras_pruning_callback_monitor_is_invalid() -> None:
 
     study = optuna.create_study(pruner=DeterministicPruner(True))
-    trial = create_running_trial(study, 1.0)
+    trial = study.ask()
     callback = KerasPruningCallback(trial, "InvalidMonitor")
 
     with pytest.warns(UserWarning):
