@@ -1,12 +1,17 @@
 from typing import Iterable
 from unittest.mock import patch
 
-from ignite.engine import Engine
 import pytest
 
 import optuna
-from optuna.testing.integration import create_running_trial
-from optuna.testing.integration import DeterministicPruner
+from optuna._imports import try_import
+from optuna.testing.pruners import DeterministicPruner
+
+
+with try_import():
+    from ignite.engine import Engine
+
+pytestmark = pytest.mark.integration
 
 
 def test_pytorch_ignite_pruning_handler() -> None:
@@ -19,7 +24,7 @@ def test_pytorch_ignite_pruning_handler() -> None:
 
     # The pruner is activated.
     study = optuna.create_study(pruner=DeterministicPruner(True))
-    trial = create_running_trial(study, 1.0)
+    trial = study.ask()
 
     handler = optuna.integration.PyTorchIgnitePruningHandler(trial, "accuracy", trainer)
     with patch.object(trainer, "state", epoch=3):
@@ -30,7 +35,7 @@ def test_pytorch_ignite_pruning_handler() -> None:
 
     # The pruner is not activated.
     study = optuna.create_study(pruner=DeterministicPruner(False))
-    trial = create_running_trial(study, 1.0)
+    trial = study.ask()
 
     handler = optuna.integration.PyTorchIgnitePruningHandler(trial, "accuracy", trainer)
     with patch.object(trainer, "state", epoch=5):
