@@ -2,18 +2,23 @@ from collections import OrderedDict
 
 import numpy as np
 import pytest
-import xgboost as xgb
 
 import optuna
+from optuna._imports import try_import
 from optuna.integration.xgboost import XGBoostPruningCallback
-from optuna.testing.integration import create_running_trial
-from optuna.testing.integration import DeterministicPruner
+from optuna.testing.pruners import DeterministicPruner
+
+
+with try_import():
+    import xgboost as xgb
+
+pytestmark = pytest.mark.integration
 
 
 def test_xgboost_pruning_callback_call() -> None:
     # The pruner is deactivated.
     study = optuna.create_study(pruner=DeterministicPruner(False))
-    trial = create_running_trial(study, 1.0)
+    trial = study.ask()
     pruning_callback = XGBoostPruningCallback(trial, "validation-logloss")
     pruning_callback.after_iteration(
         model=None, epoch=1, evals_log={"validation": OrderedDict({"logloss": [1.0]})}
@@ -21,7 +26,7 @@ def test_xgboost_pruning_callback_call() -> None:
 
     # The pruner is activated.
     study = optuna.create_study(pruner=DeterministicPruner(True))
-    trial = create_running_trial(study, 1.0)
+    trial = study.ask()
     pruning_callback = XGBoostPruningCallback(trial, "validation-logloss")
     with pytest.raises(optuna.TrialPruned):
         pruning_callback.after_iteration(
