@@ -1,4 +1,3 @@
-import itertools
 import random
 from typing import Callable
 from typing import Dict
@@ -25,60 +24,49 @@ class MockSystemAttr:
         self.value[key] = value
 
 
+def suggest(
+    sampler: optuna.samplers.BaseSampler,
+    study: optuna.Study,
+    trial: optuna.trial.FrozenTrial,
+    distribution: optuna.distributions.BaseDistribution,
+    past_trials: List[optuna.trial.FrozenTrial],
+) -> float:
+    attrs = MockSystemAttr()
+    with patch.object(study._storage, "get_all_trials", return_value=past_trials), patch.object(
+        study._storage, "set_trial_system_attr", side_effect=attrs.set_trial_system_attr
+    ), patch.object(study._storage, "get_trial", return_value=trial), patch(
+        "optuna.trial.Trial.system_attrs", new_callable=PropertyMock
+    ) as mock1, patch(
+        "optuna.trial.FrozenTrial.system_attrs",
+        new_callable=PropertyMock,
+    ) as mock2:
+        mock1.return_value = attrs.value
+        mock2.return_value = attrs.value
+        suggestion = sampler.sample_independent(study, trial, "param-a", distribution)
+    return suggestion
+
+
 def test_multi_objective_sample_independent_seed_fix() -> None:
     study = optuna.create_study(directions=["minimize", "maximize"])
-    dist = optuna.distributions.UniformDistribution(1.0, 100.0)
+    dist = optuna.distributions.FloatDistribution(1.0, 100.0)
 
     random.seed(128)
     past_trials = [frozen_trial_factory(i, [random.random(), random.random()]) for i in range(16)]
     # Prepare a trial and a sample for later checks.
     trial = frozen_trial_factory(16, [0, 0])
     sampler = TPESampler(seed=0)
-    attrs = MockSystemAttr()
-    with patch.object(study._storage, "get_all_trials", return_value=past_trials), patch.object(
-        study._storage, "set_trial_system_attr", side_effect=attrs.set_trial_system_attr
-    ), patch.object(study._storage, "get_trial", return_value=trial), patch(
-        "optuna.trial.Trial.system_attrs", new_callable=PropertyMock
-    ) as mock1, patch(
-        "optuna.trial.FrozenTrial.system_attrs",
-        new_callable=PropertyMock,
-    ) as mock2:
-        mock1.return_value = attrs.value
-        mock2.return_value = attrs.value
-        suggestion = sampler.sample_independent(study, trial, "param-a", dist)
+    suggestion = suggest(sampler, study, trial, dist, past_trials)
 
     sampler = TPESampler(seed=0)
-    attrs = MockSystemAttr()
-    with patch.object(study._storage, "get_all_trials", return_value=past_trials), patch.object(
-        study._storage, "set_trial_system_attr", side_effect=attrs.set_trial_system_attr
-    ), patch.object(study._storage, "get_trial", return_value=trial), patch(
-        "optuna.trial.Trial.system_attrs", new_callable=PropertyMock
-    ) as mock1, patch(
-        "optuna.trial.FrozenTrial.system_attrs",
-        new_callable=PropertyMock,
-    ) as mock2:
-        mock1.return_value = attrs.value
-        mock2.return_value = attrs.value
-        assert sampler.sample_independent(study, trial, "param-a", dist) == suggestion
+    assert suggest(sampler, study, trial, dist, past_trials) == suggestion
 
     sampler = TPESampler(seed=1)
-    attrs = MockSystemAttr()
-    with patch.object(study._storage, "get_all_trials", return_value=past_trials), patch.object(
-        study._storage, "set_trial_system_attr", side_effect=attrs.set_trial_system_attr
-    ), patch.object(study._storage, "get_trial", return_value=trial), patch(
-        "optuna.trial.Trial.system_attrs", new_callable=PropertyMock
-    ) as mock1, patch(
-        "optuna.trial.FrozenTrial.system_attrs",
-        new_callable=PropertyMock,
-    ) as mock2:
-        mock1.return_value = attrs.value
-        mock2.return_value = attrs.value
-        assert sampler.sample_independent(study, trial, "param-a", dist) != suggestion
+    assert suggest(sampler, study, trial, dist, past_trials) != suggestion
 
 
 def test_multi_objective_sample_independent_prior() -> None:
     study = optuna.create_study(directions=["minimize", "maximize"])
-    dist = optuna.distributions.UniformDistribution(1.0, 100.0)
+    dist = optuna.distributions.FloatDistribution(1.0, 100.0)
 
     random.seed(128)
     past_trials = [frozen_trial_factory(i, [random.random(), random.random()]) for i in range(16)]
@@ -86,270 +74,116 @@ def test_multi_objective_sample_independent_prior() -> None:
     # Prepare a trial and a sample for later checks.
     trial = frozen_trial_factory(16, [0, 0])
     sampler = TPESampler(seed=0)
-    attrs = MockSystemAttr()
-    with patch.object(study._storage, "get_all_trials", return_value=past_trials), patch.object(
-        study._storage, "set_trial_system_attr", side_effect=attrs.set_trial_system_attr
-    ), patch.object(study._storage, "get_trial", return_value=trial), patch(
-        "optuna.trial.Trial.system_attrs", new_callable=PropertyMock
-    ) as mock1, patch(
-        "optuna.trial.FrozenTrial.system_attrs",
-        new_callable=PropertyMock,
-    ) as mock2:
-        mock1.return_value = attrs.value
-        mock2.return_value = attrs.value
-        suggestion = sampler.sample_independent(study, trial, "param-a", dist)
+    suggestion = suggest(sampler, study, trial, dist, past_trials)
 
     sampler = TPESampler(consider_prior=False, seed=0)
-    attrs = MockSystemAttr()
-    with patch.object(study._storage, "get_all_trials", return_value=past_trials), patch.object(
-        study._storage, "set_trial_system_attr", side_effect=attrs.set_trial_system_attr
-    ), patch.object(study._storage, "get_trial", return_value=trial), patch(
-        "optuna.trial.Trial.system_attrs", new_callable=PropertyMock
-    ) as mock1, patch(
-        "optuna.trial.FrozenTrial.system_attrs",
-        new_callable=PropertyMock,
-    ) as mock2:
-        mock1.return_value = attrs.value
-        mock2.return_value = attrs.value
-        assert sampler.sample_independent(study, trial, "param-a", dist) != suggestion
+    assert suggest(sampler, study, trial, dist, past_trials) != suggestion
 
     sampler = TPESampler(prior_weight=0.5, seed=0)
-    attrs = MockSystemAttr()
-    with patch.object(study._storage, "get_all_trials", return_value=past_trials), patch.object(
-        study._storage, "set_trial_system_attr", side_effect=attrs.set_trial_system_attr
-    ), patch.object(study._storage, "get_trial", return_value=trial), patch(
-        "optuna.trial.Trial.system_attrs", new_callable=PropertyMock
-    ) as mock1, patch(
-        "optuna.trial.FrozenTrial.system_attrs",
-        new_callable=PropertyMock,
-    ) as mock2:
-        mock1.return_value = attrs.value
-        mock2.return_value = attrs.value
-        assert sampler.sample_independent(study, trial, "param-a", dist) != suggestion
+    assert suggest(sampler, study, trial, dist, past_trials) != suggestion
 
 
 def test_multi_objective_sample_independent_n_startup_trial() -> None:
     study = optuna.create_study(directions=["minimize", "maximize"])
-    dist = optuna.distributions.UniformDistribution(1.0, 100.0)
+    dist = optuna.distributions.FloatDistribution(1.0, 100.0)
     random.seed(128)
     past_trials = [frozen_trial_factory(i, [random.random(), random.random()]) for i in range(16)]
-
     trial = frozen_trial_factory(16, [0, 0])
-    sampler = TPESampler(n_startup_trials=16, seed=0)
-    attrs = MockSystemAttr()
-    with patch.object(
-        study._storage, "get_all_trials", return_value=past_trials[:15]
-    ), patch.object(
-        study._storage, "set_trial_system_attr", side_effect=attrs.set_trial_system_attr
-    ), patch.object(
-        study._storage, "get_trial", return_value=trial
-    ), patch(
-        "optuna.trial.Trial.system_attrs", new_callable=PropertyMock
-    ) as mock1, patch(
-        "optuna.trial.FrozenTrial.system_attrs",
-        new_callable=PropertyMock,
-    ) as mock2, patch.object(
-        optuna.samplers.RandomSampler,
-        "sample_independent",
-        return_value=1.0,
-    ) as sample_method:
-        mock1.return_value = attrs.value
-        mock2.return_value = attrs.value
-        sampler.sample_independent(study, trial, "param-a", dist)
-    assert sample_method.call_count == 1
+
+    def _suggest_and_return_call_count(
+        sampler: optuna.samplers.BaseSampler,
+        past_trials: List[optuna.trial.FrozenTrial],
+    ) -> int:
+        attrs = MockSystemAttr()
+        with patch.object(
+            study._storage, "get_all_trials", return_value=past_trials
+        ), patch.object(
+            study._storage, "set_trial_system_attr", side_effect=attrs.set_trial_system_attr
+        ), patch.object(
+            study._storage, "get_trial", return_value=trial
+        ), patch(
+            "optuna.trial.Trial.system_attrs", new_callable=PropertyMock
+        ) as mock1, patch(
+            "optuna.trial.FrozenTrial.system_attrs",
+            new_callable=PropertyMock,
+        ) as mock2, patch.object(
+            optuna.samplers.RandomSampler,
+            "sample_independent",
+            return_value=1.0,
+        ) as sample_method:
+            mock1.return_value = attrs.value
+            mock2.return_value = attrs.value
+            sampler.sample_independent(study, trial, "param-a", dist)
+        return sample_method.call_count
 
     sampler = TPESampler(n_startup_trials=16, seed=0)
-    attrs = MockSystemAttr()
-    with patch.object(study._storage, "get_all_trials", return_value=past_trials), patch.object(
-        study._storage, "set_trial_system_attr", side_effect=attrs.set_trial_system_attr
-    ), patch.object(study._storage, "get_trial", return_value=trial), patch(
-        "optuna.trial.Trial.system_attrs", new_callable=PropertyMock
-    ) as mock1, patch(
-        "optuna.trial.FrozenTrial.system_attrs",
-        new_callable=PropertyMock,
-    ) as mock2, patch.object(
-        optuna.samplers.RandomSampler,
-        "sample_independent",
-        return_value=1.0,
-    ) as sample_method:
-        mock1.return_value = attrs.value
-        mock2.return_value = attrs.value
-        sampler.sample_independent(study, trial, "param-a", dist)
-    assert sample_method.call_count == 0
+    assert _suggest_and_return_call_count(sampler, past_trials[:-1]) == 1
+
+    sampler = TPESampler(n_startup_trials=16, seed=0)
+    assert _suggest_and_return_call_count(sampler, past_trials) == 0
 
 
 def test_multi_objective_sample_independent_misc_arguments() -> None:
     study = optuna.create_study(directions=["minimize", "maximize"])
-    dist = optuna.distributions.UniformDistribution(1.0, 100.0)
+    dist = optuna.distributions.FloatDistribution(1.0, 100.0)
     random.seed(128)
     past_trials = [frozen_trial_factory(i, [random.random(), random.random()]) for i in range(32)]
 
     # Prepare a trial and a sample for later checks.
     trial = frozen_trial_factory(16, [0, 0])
     sampler = TPESampler(seed=0)
-    attrs = MockSystemAttr()
-    with patch.object(study._storage, "get_all_trials", return_value=past_trials), patch.object(
-        study._storage, "set_trial_system_attr", side_effect=attrs.set_trial_system_attr
-    ), patch.object(study._storage, "get_trial", return_value=trial), patch(
-        "optuna.trial.Trial.system_attrs", new_callable=PropertyMock
-    ) as mock1, patch(
-        "optuna.trial.FrozenTrial.system_attrs",
-        new_callable=PropertyMock,
-    ) as mock2:
-        mock1.return_value = attrs.value
-        mock2.return_value = attrs.value
-        suggestion = sampler.sample_independent(study, trial, "param-a", dist)
+    suggestion = suggest(sampler, study, trial, dist, past_trials)
 
     # Test misc. parameters.
     sampler = TPESampler(n_ei_candidates=13, seed=0)
-    attrs = MockSystemAttr()
-    with patch.object(study._storage, "get_all_trials", return_value=past_trials), patch.object(
-        study._storage, "set_trial_system_attr", side_effect=attrs.set_trial_system_attr
-    ), patch.object(study._storage, "get_trial", return_value=trial), patch(
-        "optuna.trial.Trial.system_attrs", new_callable=PropertyMock
-    ) as mock1, patch(
-        "optuna.trial.FrozenTrial.system_attrs",
-        new_callable=PropertyMock,
-    ) as mock2:
-        mock1.return_value = attrs.value
-        mock2.return_value = attrs.value
-        assert sampler.sample_independent(study, trial, "param-a", dist) != suggestion
+    assert suggest(sampler, study, trial, dist, past_trials) != suggestion
 
     sampler = TPESampler(gamma=lambda _: 1, seed=0)
-    attrs = MockSystemAttr()
-    with patch.object(study._storage, "get_all_trials", return_value=past_trials), patch.object(
-        study._storage, "set_trial_system_attr", side_effect=attrs.set_trial_system_attr
-    ), patch.object(study._storage, "get_trial", return_value=trial), patch(
-        "optuna.trial.Trial.system_attrs", new_callable=PropertyMock
-    ) as mock1, patch(
-        "optuna.trial.FrozenTrial.system_attrs",
-        new_callable=PropertyMock,
-    ) as mock2:
-        mock1.return_value = attrs.value
-        mock2.return_value = attrs.value
-        assert sampler.sample_independent(study, trial, "param-a", dist) != suggestion
-
-    sampler = TPESampler(weights=lambda n: np.zeros(n), seed=0)
-    attrs = MockSystemAttr()
-    with patch.object(study._storage, "get_all_trials", return_value=past_trials), patch.object(
-        study._storage, "set_trial_system_attr", side_effect=attrs.set_trial_system_attr
-    ), patch.object(study._storage, "get_trial", return_value=trial), patch(
-        "optuna.trial.Trial.system_attrs", new_callable=PropertyMock
-    ) as mock1, patch(
-        "optuna.trial.FrozenTrial.system_attrs",
-        new_callable=PropertyMock,
-    ) as mock2:
-        mock1.return_value = attrs.value
-        mock2.return_value = attrs.value
-        assert sampler.sample_independent(study, trial, "param-a", dist) != suggestion
+    assert suggest(sampler, study, trial, dist, past_trials) != suggestion
 
 
-def test_multi_objective_sample_independent_uniform_distributions() -> None:
-    # Prepare sample from uniform distribution for cheking other distributions.
+@pytest.mark.parametrize("log, step", [(False, None), (True, None), (False, 0.1)])
+def test_multi_objective_sample_independent_float_distributions(
+    log: bool, step: Optional[float]
+) -> None:
+    # Prepare sample from float distribution for checking other distributions.
     study = optuna.create_study(directions=["minimize", "maximize"])
     random.seed(128)
-    past_trials = [frozen_trial_factory(i, [random.random(), random.random()]) for i in range(16)]
+    float_dist = optuna.distributions.FloatDistribution(1.0, 100.0, log=log, step=step)
 
-    uni_dist = optuna.distributions.UniformDistribution(1.0, 100.0)
-    trial = frozen_trial_factory(16, [0, 0])
-    sampler = TPESampler(seed=0)
-    attrs = MockSystemAttr()
-    with patch.object(study._storage, "get_all_trials", return_value=past_trials), patch.object(
-        study._storage, "set_trial_system_attr", side_effect=attrs.set_trial_system_attr
-    ), patch.object(study._storage, "get_trial", return_value=trial), patch(
-        "optuna.trial.Trial.system_attrs", new_callable=PropertyMock
-    ) as mock1, patch(
-        "optuna.trial.FrozenTrial.system_attrs",
-        new_callable=PropertyMock,
-    ) as mock2:
-        mock1.return_value = attrs.value
-        mock2.return_value = attrs.value
-        uniform_suggestion = sampler.sample_independent(study, trial, "param-a", uni_dist)
-    assert 1.0 <= uniform_suggestion < 100.0
-
-
-def test_multi_objective_sample_independent_log_uniform_distributions() -> None:
-    """Prepare sample from uniform distribution for cheking other distributions."""
-
-    study = optuna.create_study(directions=["minimize", "maximize"])
-    random.seed(128)
-
-    uni_dist = optuna.distributions.UniformDistribution(1.0, 100.0)
-    past_trials = [frozen_trial_factory(i, [random.random(), random.random()]) for i in range(16)]
-    trial = frozen_trial_factory(16, [0, 0])
-    sampler = TPESampler(seed=0)
-    attrs = MockSystemAttr()
-    with patch.object(study._storage, "get_all_trials", return_value=past_trials), patch.object(
-        study._storage, "set_trial_system_attr", side_effect=attrs.set_trial_system_attr
-    ), patch.object(study._storage, "get_trial", return_value=trial), patch(
-        "optuna.trial.Trial.system_attrs", new_callable=PropertyMock
-    ) as mock1, patch(
-        "optuna.trial.FrozenTrial.system_attrs",
-        new_callable=PropertyMock,
-    ) as mock2:
-        mock1.return_value = attrs.value
-        mock2.return_value = attrs.value
-        uniform_suggestion = sampler.sample_independent(study, trial, "param-a", uni_dist)
-
-    # Test sample from log-uniform is different from uniform.
-    log_dist = optuna.distributions.LogUniformDistribution(1.0, 100.0)
-    past_trials = [
-        frozen_trial_factory(i, [random.random(), random.random()], log_dist) for i in range(16)
-    ]
-    trial = frozen_trial_factory(16, [0, 0])
-    sampler = TPESampler(seed=0)
-    attrs = MockSystemAttr()
-    with patch.object(study._storage, "get_all_trials", return_value=past_trials), patch.object(
-        study._storage, "set_trial_system_attr", side_effect=attrs.set_trial_system_attr
-    ), patch.object(study._storage, "get_trial", return_value=trial), patch(
-        "optuna.trial.Trial.system_attrs", new_callable=PropertyMock
-    ) as mock1, patch(
-        "optuna.trial.FrozenTrial.system_attrs",
-        new_callable=PropertyMock,
-    ) as mock2:
-        mock1.return_value = attrs.value
-        mock2.return_value = attrs.value
-        loguniform_suggestion = sampler.sample_independent(study, trial, "param-a", log_dist)
-    assert 1.0 <= loguniform_suggestion < 100.0
-    assert uniform_suggestion != loguniform_suggestion
-
-
-def test_multi_objective_sample_independent_disrete_uniform_distributions() -> None:
-    """Test samples from discrete have expected intervals."""
-
-    study = optuna.create_study(directions=["minimize", "maximize"])
-    random.seed(128)
-
-    disc_dist = optuna.distributions.DiscreteUniformDistribution(1.0, 100.0, 0.1)
-
-    def value_fn(idx: int) -> float:
-        return int(random.random() * 1000) * 0.1
+    if float_dist.step:
+        value_fn: Optional[Callable[[int], float]] = (
+            lambda number: int(random.random() * 1000) * 0.1
+        )
+    else:
+        value_fn = None
 
     past_trials = [
         frozen_trial_factory(
-            i, [random.random(), random.random()], dist=disc_dist, value_fn=value_fn
+            i, [random.random(), random.random()], dist=float_dist, value_fn=value_fn
         )
         for i in range(16)
     ]
 
     trial = frozen_trial_factory(16, [0, 0])
     sampler = TPESampler(seed=0)
-    attrs = MockSystemAttr()
-    with patch.object(study._storage, "get_all_trials", return_value=past_trials), patch.object(
-        study._storage, "set_trial_system_attr", side_effect=attrs.set_trial_system_attr
-    ), patch.object(study._storage, "get_trial", return_value=trial), patch(
-        "optuna.trial.Trial.system_attrs", new_callable=PropertyMock
-    ) as mock1, patch(
-        "optuna.trial.FrozenTrial.system_attrs",
-        new_callable=PropertyMock,
-    ) as mock2:
-        mock1.return_value = attrs.value
-        mock2.return_value = attrs.value
-        discrete_uniform_suggestion = sampler.sample_independent(
-            study, trial, "param-a", disc_dist
-        )
-    assert 1.0 <= discrete_uniform_suggestion <= 100.0
-    assert abs(int(discrete_uniform_suggestion * 10) - discrete_uniform_suggestion * 10) < 1e-3
+    float_suggestion = suggest(sampler, study, trial, float_dist, past_trials)
+    assert 1.0 <= float_suggestion < 100.0
+
+    if float_dist.step == 0.1:
+        assert abs(int(float_suggestion * 10) - float_suggestion * 10) < 1e-3
+
+    # Test sample is different when `float_dist.log` is True or float_dist.step != 1.0.
+    random.seed(128)
+    dist = optuna.distributions.FloatDistribution(1.0, 100.0)
+    past_trials = [frozen_trial_factory(i, [random.random(), random.random()]) for i in range(16)]
+    trial = frozen_trial_factory(16, [0, 0])
+    sampler = TPESampler(seed=0)
+    suggestion = suggest(sampler, study, trial, dist, past_trials)
+    if float_dist.log or float_dist.step == 0.1:
+        assert float_suggestion != suggestion
+    else:
+        assert float_suggestion == suggestion
 
 
 def test_multi_objective_sample_independent_categorical_distributions() -> None:
@@ -373,31 +207,28 @@ def test_multi_objective_sample_independent_categorical_distributions() -> None:
 
     trial = frozen_trial_factory(16, [0, 0])
     sampler = TPESampler(seed=0)
-    attrs = MockSystemAttr()
-    with patch.object(study._storage, "get_all_trials", return_value=past_trials), patch.object(
-        study._storage, "set_trial_system_attr", side_effect=attrs.set_trial_system_attr
-    ), patch.object(study._storage, "get_trial", return_value=trial), patch(
-        "optuna.trial.Trial.system_attrs", new_callable=PropertyMock
-    ) as mock1, patch(
-        "optuna.trial.FrozenTrial.system_attrs",
-        new_callable=PropertyMock,
-    ) as mock2:
-        mock1.return_value = attrs.value
-        mock2.return_value = attrs.value
-        categorical_suggestion = sampler.sample_independent(study, trial, "param-a", cat_dist)
+    categorical_suggestion = suggest(sampler, study, trial, cat_dist, past_trials)
     assert categorical_suggestion in categories
 
 
-def test_multi_objective_sample_int_uniform_distributions() -> None:
+@pytest.mark.parametrize(
+    "log, step",
+    [
+        (False, 1),
+        (True, 1),
+        (False, 2),
+    ],
+)
+def test_multi_objective_sample_int_distributions(log: bool, step: int) -> None:
     """Test sampling from int distribution returns integer."""
 
     study = optuna.create_study(directions=["minimize", "maximize"])
     random.seed(128)
 
     def int_value_fn(idx: int) -> float:
-        return random.randint(0, 100)
+        return random.randint(1, 99)
 
-    int_dist = optuna.distributions.IntUniformDistribution(1, 100)
+    int_dist = optuna.distributions.IntDistribution(1, 99, log, step)
     past_trials = [
         frozen_trial_factory(
             i, [random.random(), random.random()], dist=int_dist, value_fn=int_value_fn
@@ -407,19 +238,8 @@ def test_multi_objective_sample_int_uniform_distributions() -> None:
 
     trial = frozen_trial_factory(16, [0, 0])
     sampler = TPESampler(seed=0)
-    attrs = MockSystemAttr()
-    with patch.object(study._storage, "get_all_trials", return_value=past_trials), patch.object(
-        study._storage, "set_trial_system_attr", side_effect=attrs.set_trial_system_attr
-    ), patch.object(study._storage, "get_trial", return_value=trial), patch(
-        "optuna.trial.Trial.system_attrs", new_callable=PropertyMock
-    ) as mock1, patch(
-        "optuna.trial.FrozenTrial.system_attrs",
-        new_callable=PropertyMock,
-    ) as mock2:
-        mock1.return_value = attrs.value
-        mock2.return_value = attrs.value
-        int_suggestion = sampler.sample_independent(study, trial, "param-a", int_dist)
-    assert 1 <= int_suggestion <= 100
+    int_suggestion = suggest(sampler, study, trial, int_dist, past_trials)
+    assert 1 <= int_suggestion <= 99
     assert isinstance(int_suggestion, int)
 
 
@@ -436,7 +256,7 @@ def test_multi_objective_sample_independent_handle_unsuccessful_states(
     state: optuna.trial.TrialState,
 ) -> None:
     study = optuna.create_study(directions=["minimize", "maximize"])
-    dist = optuna.distributions.UniformDistribution(1.0, 100.0)
+    dist = optuna.distributions.FloatDistribution(1.0, 100.0)
     random.seed(128)
 
     # Prepare sampling result for later tests.
@@ -444,18 +264,7 @@ def test_multi_objective_sample_independent_handle_unsuccessful_states(
 
     trial = frozen_trial_factory(32, [0, 0])
     sampler = TPESampler(seed=0)
-    attrs = MockSystemAttr()
-    with patch.object(study._storage, "get_all_trials", return_value=past_trials), patch.object(
-        study._storage, "set_trial_system_attr", side_effect=attrs.set_trial_system_attr
-    ), patch.object(study._storage, "get_trial", return_value=trial), patch(
-        "optuna.trial.Trial.system_attrs", new_callable=PropertyMock
-    ) as mock1, patch(
-        "optuna.trial.FrozenTrial.system_attrs",
-        new_callable=PropertyMock,
-    ) as mock2:
-        mock1.return_value = attrs.value
-        mock2.return_value = attrs.value
-        all_success_suggestion = sampler.sample_independent(study, trial, "param-a", dist)
+    all_success_suggestion = suggest(sampler, study, trial, dist, past_trials)
 
     # Test unsuccessful trials are handled differently.
     state_fn = build_state_fn(state)
@@ -466,25 +275,14 @@ def test_multi_objective_sample_independent_handle_unsuccessful_states(
 
     trial = frozen_trial_factory(32, [0, 0])
     sampler = TPESampler(seed=0)
-    attrs = MockSystemAttr()
-    with patch.object(study._storage, "get_all_trials", return_value=past_trials), patch.object(
-        study._storage, "set_trial_system_attr", side_effect=attrs.set_trial_system_attr
-    ), patch.object(study._storage, "get_trial", return_value=trial), patch(
-        "optuna.trial.Trial.system_attrs", new_callable=PropertyMock
-    ) as mock1, patch(
-        "optuna.trial.FrozenTrial.system_attrs",
-        new_callable=PropertyMock,
-    ) as mock2:
-        mock1.return_value = attrs.value
-        mock2.return_value = attrs.value
-        partial_unsuccessful_suggestion = sampler.sample_independent(study, trial, "param-a", dist)
+    partial_unsuccessful_suggestion = suggest(sampler, study, trial, dist, past_trials)
     assert partial_unsuccessful_suggestion != all_success_suggestion
 
 
 def test_multi_objective_sample_independent_ignored_states() -> None:
     """Tests FAIL, RUNNING, and WAITING states are equally."""
     study = optuna.create_study(directions=["minimize", "maximize"])
-    dist = optuna.distributions.UniformDistribution(1.0, 100.0)
+    dist = optuna.distributions.FloatDistribution(1.0, 100.0)
 
     suggestions = []
     for state in [
@@ -500,44 +298,116 @@ def test_multi_objective_sample_independent_ignored_states() -> None:
         ]
         trial = frozen_trial_factory(32, [0, 0])
         sampler = TPESampler(seed=0)
-    attrs = MockSystemAttr()
-    with patch.object(study._storage, "get_all_trials", return_value=past_trials), patch.object(
-        study._storage, "set_trial_system_attr", side_effect=attrs.set_trial_system_attr
-    ), patch.object(study._storage, "get_trial", return_value=trial), patch(
-        "optuna.trial.Trial.system_attrs", new_callable=PropertyMock
-    ) as mock1, patch(
-        "optuna.trial.FrozenTrial.system_attrs",
-        new_callable=PropertyMock,
-    ) as mock2:
-        mock1.return_value = attrs.value
-        mock2.return_value = attrs.value
-        suggestions.append(sampler.sample_independent(study, trial, "param-a", dist))
+        suggestions.append(suggest(sampler, study, trial, dist, past_trials))
 
     assert len(set(suggestions)) == 1
 
 
-def test_multi_objective_get_observation_pairs() -> None:
+@pytest.mark.parametrize("int_value", [-5, 5, 0])
+@pytest.mark.parametrize(
+    "categorical_value", [1, 0.0, "A", None, True, float("inf"), float("nan")]
+)
+@pytest.mark.parametrize("objective_value", [-5.0, 5.0, 0.0, -float("inf"), float("inf")])
+@pytest.mark.parametrize("multivariate", [True, False])
+@pytest.mark.parametrize("constant_liar", [True, False])
+@pytest.mark.filterwarnings("ignore::optuna.exceptions.ExperimentalWarning")
+def test_multi_objective_get_observation_pairs(
+    int_value: int,
+    categorical_value: optuna.distributions.CategoricalChoiceType,
+    objective_value: float,
+    multivariate: bool,
+    constant_liar: bool,
+) -> None:
+    def objective(trial: optuna.trial.Trial) -> Tuple[float, float]:
+        trial.suggest_int("x", int_value, int_value)
+        trial.suggest_categorical("y", [categorical_value])
+        return objective_value, objective_value
+
+    sampler = TPESampler(seed=0, multivariate=multivariate, constant_liar=constant_liar)
+    study = optuna.create_study(directions=["minimize", "maximize"], sampler=sampler)
+    study.optimize(objective, n_trials=2)
+    study.add_trial(
+        optuna.create_trial(
+            state=optuna.trial.TrialState.RUNNING,
+            params={"x": int_value, "y": categorical_value},
+            distributions={
+                "x": optuna.distributions.IntDistribution(int_value, int_value),
+                "y": optuna.distributions.CategoricalDistribution([categorical_value]),
+            },
+        )
+    )
+
+    assert _tpe.sampler._get_observation_pairs(study, ["x"], constant_liar) == (
+        {"x": [int_value, int_value]},
+        [(-float("inf"), [objective_value, -objective_value]) for _ in range(2)],
+        None,
+    )
+    assert _tpe.sampler._get_observation_pairs(study, ["y"], constant_liar) == (
+        {"y": [0, 0]},
+        [(-float("inf"), [objective_value, -objective_value]) for _ in range(2)],
+        None,
+    )
+    assert _tpe.sampler._get_observation_pairs(study, ["x", "y"], constant_liar) == (
+        {"x": [int_value, int_value], "y": [0, 0]},
+        [(-float("inf"), [objective_value, -objective_value]) for _ in range(2)],
+        None,
+    )
+    assert _tpe.sampler._get_observation_pairs(study, ["z"], constant_liar) == (
+        {"z": [None, None]},
+        [(-float("inf"), [objective_value, -objective_value]) for _ in range(2)],
+        None,
+    )
+
+
+@pytest.mark.parametrize("constraint_value", [-2, 2])
+def test_multi_objective_get_observation_pairs_constrained(constraint_value: int) -> None:
     def objective(trial: optuna.trial.Trial) -> Tuple[float, float]:
         trial.suggest_int("x", 5, 5)
+        trial.set_user_attr("constraint", (constraint_value, -1))
         return 5.0, 5.0
 
-    sampler = TPESampler(seed=0)
+    sampler = TPESampler(constraints_func=lambda trial: trial.user_attrs["constraint"], seed=0)
     study = optuna.create_study(directions=["minimize", "maximize"], sampler=sampler)
     study.optimize(objective, n_trials=5)
 
-    assert _tpe.sampler._get_observation_pairs(study, ["x"], False) == (
+    violations = [max(0, constraint_value) for _ in range(5)]
+    assert _tpe.sampler._get_observation_pairs(study, ["x"], constraints_enabled=True) == (
         {"x": [5.0, 5.0, 5.0, 5.0, 5.0]},
         [(-float("inf"), [5.0, -5.0]) for _ in range(5)],
+        violations,
     )
-    assert _tpe.sampler._get_observation_pairs(study, ["y"], False) == (
+    assert _tpe.sampler._get_observation_pairs(study, ["y"], constraints_enabled=True) == (
         {"y": [None, None, None, None, None]},
         [(-float("inf"), [5.0, -5.0]) for _ in range(5)],
+        violations,
     )
-    assert _tpe.sampler._get_observation_pairs(study, ["x"], True) == (
-        {"x": [5.0, 5.0, 5.0, 5.0, 5.0]},
-        [(-float("inf"), [5.0, -5.0]) for _ in range(5)],
+
+
+def test_multi_objective_split_observation_pairs() -> None:
+    indices_below, indices_above = _tpe.sampler._split_observation_pairs(
+        [
+            (-float("inf"), [-2.0, -1.0]),
+            (-float("inf"), [3.0, 3.0]),
+            (-float("inf"), [0.0, 1.0]),
+            (-float("inf"), [-1.0, 0.0]),
+        ],
+        2,
+        None,
     )
-    assert _tpe.sampler._get_observation_pairs(study, ["y"], True) == ({"y": []}, [])
+    assert list(indices_below) == [0, 3]
+    assert list(indices_above) == [1, 2]
+
+
+def test_multi_objective_split_observation_pairs_with_all_indices_below() -> None:
+    indices_below, indices_above = _tpe.sampler._split_observation_pairs(
+        [
+            (-float("inf"), [1.0, 1.0]),
+        ],
+        1,
+        None,
+    )
+    assert list(indices_below) == [0]
+    assert list(indices_above) == []
 
 
 def test_calculate_nondomination_rank() -> None:
@@ -556,13 +426,51 @@ def test_calculate_nondomination_rank() -> None:
     ranks = list(_tpe.sampler._calculate_nondomination_rank(test_case))
     assert ranks == [0, 1, 0, 2, 0, 1]
 
+    # The negative values are included.
+    test_case = np.asarray(
+        [[-5, -5, -4], [-5, -5, 5], [-9, -9, 0], [5, 7, 5], [0, 0, -9], [0, -9, 9]]
+    )
+    ranks = list(_tpe.sampler._calculate_nondomination_rank(test_case))
+    assert ranks == [0, 1, 0, 2, 0, 1]
+
+    # The +inf is included.
+    test_case = np.asarray(
+        [[1, 1], [1, float("inf")], [float("inf"), 1], [float("inf"), float("inf")]]
+    )
+    ranks = list(_tpe.sampler._calculate_nondomination_rank(test_case))
+    assert ranks == [0, 1, 1, 2]
+
+    # The -inf is included.
+    test_case = np.asarray(
+        [[1, 1], [1, -float("inf")], [-float("inf"), 1], [-float("inf"), -float("inf")]]
+    )
+    ranks = list(_tpe.sampler._calculate_nondomination_rank(test_case))
+    assert ranks == [2, 1, 1, 0]
+
 
 def test_calculate_weights_below_for_multi_objective() -> None:
+    # No sample.
+    weights_below = _tpe.sampler._calculate_weights_below_for_multi_objective(
+        [(0, [0.2, 0.5]), (0, [0.9, 0.4]), (0, [1, 1])],
+        np.array([], np.int64),
+        None,
+    )
+    assert len(weights_below) == 0
+
+    # One sample.
+    weights_below = _tpe.sampler._calculate_weights_below_for_multi_objective(
+        [(0, [0.2, 0.5]), (0, [0.9, 0.4]), (0, [1, 1])],
+        np.array([0]),
+        None,
+    )
+    assert len(weights_below) == 1
+    assert sum(weights_below) > 0
+
     # Two samples.
     weights_below = _tpe.sampler._calculate_weights_below_for_multi_objective(
-        {"x": np.array([1.0, 2.0, 3.0], dtype=float)},
         [(0, [0.2, 0.5]), (0, [0.9, 0.4]), (0, [1, 1])],
         np.array([0, 1]),
+        None,
     )
     assert len(weights_below) == 2
     assert weights_below[0] > weights_below[1]
@@ -570,9 +478,9 @@ def test_calculate_weights_below_for_multi_objective() -> None:
 
     # Two equally contributed samples.
     weights_below = _tpe.sampler._calculate_weights_below_for_multi_objective(
-        {"x": np.array([1.0, 2.0, 3.0], dtype=float)},
         [(0, [0.2, 0.8]), (0, [0.8, 0.2]), (0, [1, 1])],
         np.array([0, 1]),
+        None,
     )
     assert len(weights_below) == 2
     assert weights_below[0] == weights_below[1]
@@ -580,9 +488,9 @@ def test_calculate_weights_below_for_multi_objective() -> None:
 
     # Duplicated samples.
     weights_below = _tpe.sampler._calculate_weights_below_for_multi_objective(
-        {"x": np.array([1.0, 2.0, 3.0], dtype=float)},
         [(0, [0.2, 0.8]), (0, [0.2, 0.8]), (0, [1, 1])],
         np.array([0, 1]),
+        None,
     )
     assert len(weights_below) == 2
     assert weights_below[0] == weights_below[1]
@@ -590,9 +498,9 @@ def test_calculate_weights_below_for_multi_objective() -> None:
 
     # Three samples.
     weights_below = _tpe.sampler._calculate_weights_below_for_multi_objective(
-        {"x": np.array([1.0, 2.0, 3.0, 4.0], dtype=float)},
         [(0, [0.3, 0.3]), (0, [0.2, 0.8]), (0, [0.8, 0.2]), (0, [1, 1])],
         np.array([0, 1, 2]),
+        None,
     )
     assert len(weights_below) == 3
     assert weights_below[0] > weights_below[1]
@@ -600,41 +508,48 @@ def test_calculate_weights_below_for_multi_objective() -> None:
     assert weights_below[1] == weights_below[2]
     assert sum(weights_below) > 0
 
+    # Zero/negative objective values.
+    weights_below = _tpe.sampler._calculate_weights_below_for_multi_objective(
+        [(0, [-0.3, -0.3]), (0, [0.0, -0.8]), (0, [-0.8, 0.0]), (0, [1, 1])],
+        np.array([0, 1, 2]),
+        None,
+    )
+    assert len(weights_below) == 3
+    assert weights_below[0] > weights_below[1]
+    assert weights_below[0] > weights_below[2]
+    assert np.isclose(weights_below[1], weights_below[2])
+    assert sum(weights_below) > 0
 
-def test_solve_hssp() -> None:
-    random.seed(128)
+    # +/-inf objective values.
+    weights_below = _tpe.sampler._calculate_weights_below_for_multi_objective(
+        [
+            (0, [-float("inf"), -float("inf")]),
+            (0, [0.0, -float("inf")]),
+            (0, [-float("inf"), 0.0]),
+            (0, [float("inf"), float("inf")]),
+        ],
+        np.array([0, 1, 2]),
+        None,
+    )
+    assert len(weights_below) == 3
+    assert all([np.isnan(w) for w in weights_below])
 
-    # Two dimensions
-    for i in range(8):
-        subset_size = int(random.random() * i) + 1
-        test_case = np.asarray([[random.random(), random.random()] for _ in range(8)])
-        r = 1.1 * np.max(test_case, axis=0)
-        truth = 0.0
-        for subset in itertools.permutations(test_case, subset_size):
-            truth = max(truth, _tpe.sampler._compute_hypervolume(np.asarray(subset), r))
-        indices = _tpe.sampler._solve_hssp(test_case, np.arange(len(test_case)), subset_size, r)
-        approx = _tpe.sampler._compute_hypervolume(test_case[indices], r)
-        assert approx / truth > 0.6321  # 1 - 1/e
-
-    # Three dimensions
-    for i in range(8):
-        subset_size = int(random.random() * i) + 1
-        test_case = np.asarray(
-            [[random.random(), random.random(), random.random()] for _ in range(8)]
-        )
-        r = 1.1 * np.max(test_case, axis=0)
-        truth = 0
-        for subset in itertools.permutations(test_case, subset_size):
-            truth = max(truth, _tpe.sampler._compute_hypervolume(np.asarray(subset), r))
-        indices = _tpe.sampler._solve_hssp(test_case, np.arange(len(test_case)), subset_size, r)
-        approx = _tpe.sampler._compute_hypervolume(test_case[indices], r)
-        assert approx / truth > 0.6321  # 1 - 1/e
+    # Three samples with two infeasible trials.
+    weights_below = _tpe.sampler._calculate_weights_below_for_multi_objective(
+        [(0, [0.3, 0.3]), (0, [0.2, 0.8]), (0, [0.8, 0.2]), (0, [1, 1])],
+        np.array([0, 1, 2]),
+        [2, 8, 0],
+    )
+    assert len(weights_below) == 3
+    assert weights_below[0] == _tpe.sampler.EPS
+    assert weights_below[1] == _tpe.sampler.EPS
+    assert weights_below[2] > 0
 
 
 def frozen_trial_factory(
     number: int,
     values: List[float],
-    dist: optuna.distributions.BaseDistribution = optuna.distributions.UniformDistribution(
+    dist: optuna.distributions.BaseDistribution = optuna.distributions.FloatDistribution(
         1.0, 100.0
     ),
     value_fn: Optional[Callable[[int], Union[int, float]]] = None,
