@@ -28,7 +28,7 @@ def session() -> Session:
     return Session(bind=engine)
 
 
-class TestStudyDirectionModel(object):
+class TestStudyDirectionModel:
     @staticmethod
     def _create_model(session: Session) -> StudyModel:
 
@@ -47,16 +47,6 @@ class TestStudyDirectionModel(object):
         session.commit()
         return study
 
-    @staticmethod
-    def test_find_by_study_and_objective(session: Session) -> None:
-
-        study = TestStudyDirectionModel._create_model(session)
-        direction = StudyDirectionModel.find_by_study_and_objective(study, 0, session)
-        assert direction is not None
-        assert direction.direction == StudyDirection.MINIMIZE
-
-        assert StudyDirectionModel.find_by_study_and_objective(study, 1, session) is None
-
     @staticmethod
     def test_where_study_id(session: Session) -> None:
 
@@ -83,7 +73,7 @@ class TestStudyDirectionModel(object):
         assert 0 == len(StudyDirectionModel.where_study_id(study.study_id, session))
 
 
-class TestStudySystemAttributeModel(object):
+class TestStudySystemAttributeModel:
     @staticmethod
     def test_find_by_study_and_key(session: Session) -> None:
 
@@ -138,7 +128,7 @@ class TestStudySystemAttributeModel(object):
         assert 0 == len(StudySystemAttributeModel.where_study_id(study_id, session))
 
 
-class TestTrialModel(object):
+class TestTrialModel:
     @staticmethod
     def test_default_datetime(session: Session) -> None:
         # Regardless of the initial state the trial created here should have null datetime_start
@@ -205,7 +195,7 @@ class TestTrialModel(object):
         assert 0 == TrialModel.count(session, study)
 
 
-class TestTrialUserAttributeModel(object):
+class TestTrialUserAttributeModel:
     @staticmethod
     def test_find_by_trial_and_key(session: Session) -> None:
 
@@ -247,7 +237,7 @@ class TestTrialUserAttributeModel(object):
         assert 0 == len(TrialUserAttributeModel.where_trial_id(trial_id, session))
 
 
-class TestTrialSystemAttributeModel(object):
+class TestTrialSystemAttributeModel:
     @staticmethod
     def test_find_by_trial_and_key(session: Session) -> None:
 
@@ -289,7 +279,7 @@ class TestTrialSystemAttributeModel(object):
         assert 0 == len(TrialSystemAttributeModel.where_trial_id(trial_id, session))
 
 
-class TestTrialValueModel(object):
+class TestTrialValueModel:
     @staticmethod
     def _create_model(session: Session) -> TrialModel:
 
@@ -298,7 +288,14 @@ class TestTrialValueModel(object):
         trial = TrialModel(trial_id=1, study_id=study.study_id, state=TrialState.COMPLETE)
         session.add(study)
         session.add(trial)
-        session.add(TrialValueModel(trial_id=trial.trial_id, objective=0, value=10))
+        session.add(
+            TrialValueModel(
+                trial_id=trial.trial_id,
+                objective=0,
+                value=10,
+                value_type=TrialValueModel.TrialValueType.FINITE,
+            )
+        )
         session.commit()
         return trial
 
@@ -324,7 +321,11 @@ class TestTrialValueModel(object):
     def test_cascade_delete_on_trial(session: Session) -> None:
 
         trial = TestTrialValueModel._create_model(session)
-        trial.values.append(TrialValueModel(trial_id=1, objective=1, value=20))
+        trial.values.append(
+            TrialValueModel(
+                trial_id=1, objective=1, value=20, value_type=TrialValueModel.TrialValueType.FINITE
+            )
+        )
         session.commit()
 
         assert 2 == len(TrialValueModel.where_trial_id(trial.trial_id, session))
@@ -335,7 +336,7 @@ class TestTrialValueModel(object):
         assert 0 == len(TrialValueModel.where_trial_id(trial.trial_id, session))
 
 
-class TestTrialIntermediateValueModel(object):
+class TestTrialIntermediateValueModel:
     @staticmethod
     def _create_model(session: Session) -> TrialModel:
 
@@ -345,7 +346,12 @@ class TestTrialIntermediateValueModel(object):
         session.add(study)
         session.add(trial)
         session.add(
-            TrialIntermediateValueModel(trial_id=trial.trial_id, step=0, intermediate_value=10)
+            TrialIntermediateValueModel(
+                trial_id=trial.trial_id,
+                step=0,
+                intermediate_value=10,
+                intermediate_value_type=TrialIntermediateValueModel.TrialIntermediateValueType.FINITE,  # noqa: E501
+            )
         )
         session.commit()
         return trial
@@ -377,7 +383,12 @@ class TestTrialIntermediateValueModel(object):
 
         trial = TestTrialIntermediateValueModel._create_model(session)
         trial.intermediate_values.append(
-            TrialIntermediateValueModel(trial_id=1, step=1, intermediate_value=20)
+            TrialIntermediateValueModel(
+                trial_id=1,
+                step=1,
+                intermediate_value=20,
+                intermediate_value_type=TrialIntermediateValueModel.TrialIntermediateValueType.FINITE,  # noqa: E501
+            )
         )
         session.commit()
 
@@ -389,7 +400,7 @@ class TestTrialIntermediateValueModel(object):
         assert 0 == len(TrialIntermediateValueModel.where_trial_id(trial.trial_id, session))
 
 
-class TestTrialHeartbeatModel(object):
+class TestTrialHeartbeatModel:
     @staticmethod
     def _create_model(session: Session) -> TrialModel:
 
@@ -424,7 +435,7 @@ class TestTrialHeartbeatModel(object):
         assert TrialHeartbeatModel.where_trial_id(trial.trial_id, session) is None
 
 
-class TestVersionInfoModel(object):
+class TestVersionInfoModel:
     @staticmethod
     def test_version_info_id_constraint(session: Session) -> None:
 
