@@ -3,6 +3,7 @@ import copy
 import pytest
 
 from optuna import create_study
+from optuna import get_all_study_summaries
 from optuna.storages import RDBStorage
 
 
@@ -13,7 +14,7 @@ def test_study_summary_eq_ne() -> None:
     create_study(storage=storage)
     study = create_study(storage=storage)
 
-    summaries = study._storage.get_all_study_summaries()
+    summaries = get_all_study_summaries(study._storage, include_best_trial=True)
     assert len(summaries) == 2
 
     assert summaries[0] == copy.deepcopy(summaries[0])
@@ -33,7 +34,7 @@ def test_study_summary_lt_le() -> None:
     create_study(storage=storage)
     study = create_study(storage=storage)
 
-    summaries = study._storage.get_all_study_summaries()
+    summaries = get_all_study_summaries(study._storage, include_best_trial=True)
     assert len(summaries) == 2
 
     summary_0 = summaries[0]
