@@ -1,14 +1,15 @@
 from collections import OrderedDict
 import json
+import os
 import re
 import subprocess
 from subprocess import CalledProcessError
 import tempfile
 from typing import Any
 from typing import Callable
-from typing import List
 from typing import Optional
 from typing import Tuple
+from unittest.mock import patch
 
 import numpy as np
 from pandas import Timedelta
@@ -19,10 +20,11 @@ import yaml
 import optuna
 import optuna.cli
 from optuna.exceptions import CLIUsageError
+from optuna.exceptions import ExperimentalWarning
 from optuna.storages import RDBStorage
 from optuna.storages._base import DEFAULT_STUDY_NAME_PREFIX
 from optuna.study import StudyDirection
-from optuna.testing.storage import StorageSupplier
+from optuna.testing.storages import StorageSupplier
 from optuna.trial import Trial
 from optuna.trial import TrialState
 
@@ -89,6 +91,7 @@ def _parse_output(output: str, output_format: str) -> Any:
         assert False
 
 
+@pytest.mark.skip_coverage
 def test_create_study_command() -> None:
 
     with StorageSupplier("sqlite") as storage:
@@ -109,6 +112,7 @@ def test_create_study_command() -> None:
         assert study_id == 2
 
 
+@pytest.mark.skip_coverage
 def test_create_study_command_with_study_name() -> None:
 
     with StorageSupplier("sqlite") as storage:
@@ -125,14 +129,41 @@ def test_create_study_command_with_study_name() -> None:
         assert storage.get_study_name_from_id(study_id) == study_name
 
 
+@pytest.mark.skip_coverage
 def test_create_study_command_without_storage_url() -> None:
 
     with pytest.raises(subprocess.CalledProcessError) as err:
-        subprocess.check_output(["optuna", "create-study"])
+        subprocess.check_output(
+            ["optuna", "create-study"],
+            env={k: v for k, v in os.environ.items() if k != "OPTUNA_STORAGE"},
+        )
     usage = err.value.output.decode()
     assert usage.startswith("usage:")
 
 
+@pytest.mark.skip_coverage
+def test_create_study_command_with_storage_env() -> None:
+
+    with StorageSupplier("sqlite") as storage:
+        assert isinstance(storage, RDBStorage)
+        storage_url = str(storage.engine.url)
+
+        # Create study.
+        command = ["optuna", "create-study"]
+        env = {**os.environ, "OPTUNA_STORAGE": storage_url}
+        subprocess.check_call(command, env=env)
+
+        # Command output should be in name string format (no-name + UUID).
+        study_name = str(subprocess.check_output(command, env=env).decode().strip())
+        name_re = r"^no-name-[a-f0-9]{8}-[a-f0-9]{4}-[a-f0-9]{4}-[a-f0-9]{4}-[a-f0-9]{12}$"
+        assert re.match(name_re, study_name) is not None
+
+        # study_name should be stored in storage.
+        study_id = storage.get_study_id_from_name(study_name)
+        assert study_id == 2
+
+
+@pytest.mark.skip_coverage
 def test_create_study_command_with_direction() -> None:
 
     with StorageSupplier("sqlite") as storage:
@@ -156,6 +187,7 @@ def test_create_study_command_with_direction() -> None:
             subprocess.check_call(command)
 
 
+@pytest.mark.skip_coverage
 def test_create_study_command_with_multiple_directions() -> None:
 
     with StorageSupplier("sqlite") as storage:
@@ -209,6 +241,7 @@ def test_create_study_command_with_multiple_directions() -> None:
             subprocess.check_call(command)
 
 
+@pytest.mark.skip_coverage
 def test_delete_study_command() -> None:
 
     with StorageSupplier("sqlite") as storage:
@@ -219,20 +252,25 @@ def test_delete_study_command() -> None:
         # Create study.
         command = ["optuna", "create-study", "--storage", storage_url, "--study-name", study_name]
         subprocess.check_call(command)
-        assert study_name in {s.study_name: s for s in storage.get_all_study_summaries()}
+        assert study_name in {s.study_name: s for s in storage.get_all_studies()}
 
         # Delete study.
         command = ["optuna", "delete-study", "--storage", storage_url, "--study-name", study_name]
         subprocess.check_call(command)
-        assert study_name not in {s.study_name: s for s in storage.get_all_study_summaries()}
+        assert study_name not in {s.study_name: s for s in storage.get_all_studies()}
 
 
+@pytest.mark.skip_coverage
 def test_delete_study_command_without_storage_url() -> None:
 
     with pytest.raises(subprocess.CalledProcessError):
-        subprocess.check_output(["optuna", "delete-study", "--study-name", "dummy_study"])
+        subprocess.check_output(
+            ["optuna", "delete-study", "--study-name", "dummy_study"],
+            env={k: v for k, v in os.environ.items() if k != "OPTUNA_STORAGE"},
+        )
 
 
+@pytest.mark.skip_coverage
 def test_study_set_user_attr_command() -> None:
 
     with StorageSupplier("sqlite") as storage:
@@ -240,7 +278,9 @@ def test_study_set_user_attr_command() -> None:
         storage_url = str(storage.engine.url)
 
         # Create study.
-        study_name = storage.get_study_name_from_id(storage.create_new_study())
+        study_name = storage.get_study_name_from_id(
+            storage.create_new_study(directions=[StudyDirection.MINIMIZE])
+        )
 
         base_command = [
             "optuna",
@@ -263,6 +303,7 @@ def test_study_set_user_attr_command() -> None:
         assert all(study_user_attrs[k] == v for k, v in example_attrs.items())
 
 
+@pytest.mark.skip_coverage
 @pytest.mark.parametrize("output_format", (None, "table", "json", "yaml"))
 def test_studies_command(output_format: Optional[str]) -> None:
 
@@ -271,13 +312,7 @@ def test_studies_command(output_format: Optional[str]) -> None:
         storage_url = str(storage.engine.url)
 
         # First study.
-        study_1 = optuna.create_study(storage)
-
-        # Second study.
-        study_2 = optuna.create_study(
-            storage, study_name="study_2", directions=["minimize", "maximize"]
-        )
-        study_2.optimize(objective_func_multi_objective, n_trials=10)
+        study_1 = optuna.create_study(storage=storage)
 
         # Run command.
         command = ["optuna", "studies", "--storage", storage_url]
@@ -289,6 +324,26 @@ def test_studies_command(output_format: Optional[str]) -> None:
 
         expected_keys = ["name", "direction", "n_trials", "datetime_start"]
 
+        # Check user_attrs are not printed.
+        if output_format is None or output_format == "table":
+            assert list(studies[0].keys()) == expected_keys
+        else:
+            assert set(studies[0].keys()) == set(expected_keys)
+
+        # Add a second study.
+        study_2 = optuna.create_study(
+            storage=storage, study_name="study_2", directions=["minimize", "maximize"]
+        )
+        study_2.optimize(objective_func_multi_objective, n_trials=10)
+        study_2.set_user_attr("key_1", "value_1")
+        study_2.set_user_attr("key_2", "value_2")
+
+        # Run command again to include second study.
+        output = str(subprocess.check_output(command).decode().strip())
+        studies = _parse_output(output, output_format or "table")
+
+        expected_keys = ["name", "direction", "n_trials", "datetime_start", "user_attrs"]
+
         assert len(studies) == 2
         for study in studies:
             if output_format is None or output_format == "table":
@@ -296,25 +351,30 @@ def test_studies_command(output_format: Optional[str]) -> None:
             else:
                 assert set(study.keys()) == set(expected_keys)
 
-        # Check study_name, direction, and n_trials for the first study.
+        # Check study_name, direction, n_trials and user_attrs for the first study.
         assert studies[0]["name"] == study_1.study_name
         if output_format is None or output_format == "table":
             assert studies[0]["n_trials"] == "0"
             assert eval(studies[0]["direction"]) == ("MINIMIZE",)
+            assert eval(studies[0]["user_attrs"]) == {}
         else:
             assert studies[0]["n_trials"] == 0
             assert studies[0]["direction"] == ["MINIMIZE"]
+            assert studies[0]["user_attrs"] == {}
 
-        # Check study_name, direction, and n_trials for the second study.
+        # Check study_name, direction, n_trials and user_attrs for the second study.
         assert studies[1]["name"] == study_2.study_name
         if output_format is None or output_format == "table":
             assert studies[1]["n_trials"] == "10"
             assert eval(studies[1]["direction"]) == ("MINIMIZE", "MAXIMIZE")
+            assert eval(studies[1]["user_attrs"]) == {"key_1": "value_1", "key_2": "value_2"}
         else:
             assert studies[1]["n_trials"] == 10
             assert studies[1]["direction"] == ["MINIMIZE", "MAXIMIZE"]
+            assert studies[1]["user_attrs"] == {"key_1": "value_1", "key_2": "value_2"}
 
 
+@pytest.mark.skip_coverage
 @pytest.mark.parametrize("output_format", (None, "table", "json", "yaml"))
 def test_studies_command_flatten(output_format: Optional[str]) -> None:
 
@@ -323,13 +383,7 @@ def test_studies_command_flatten(output_format: Optional[str]) -> None:
         storage_url = str(storage.engine.url)
 
         # First study.
-        study_1 = optuna.create_study(storage)
-
-        # Second study.
-        study_2 = optuna.create_study(
-            storage, study_name="study_2", directions=["minimize", "maximize"]
-        )
-        study_2.optimize(objective_func_multi_objective, n_trials=10)
+        study_1 = optuna.create_study(storage=storage)
 
         # Run command.
         command = ["optuna", "studies", "--storage", storage_url, "--flatten"]
@@ -339,6 +393,31 @@ def test_studies_command_flatten(output_format: Optional[str]) -> None:
         output = str(subprocess.check_output(command).decode().strip())
         studies = _parse_output(output, output_format or "table")
 
+        expected_keys_1 = [
+            "name",
+            "direction_0",
+            "n_trials",
+            "datetime_start",
+        ]
+
+        # Check user_attrs are not printed.
+        if output_format is None or output_format == "table":
+            assert list(studies[0].keys()) == expected_keys_1
+        else:
+            assert set(studies[0].keys()) == set(expected_keys_1)
+
+        # Add a second study.
+        study_2 = optuna.create_study(
+            storage=storage, study_name="study_2", directions=["minimize", "maximize"]
+        )
+        study_2.optimize(objective_func_multi_objective, n_trials=10)
+        study_2.set_user_attr("key_1", "value_1")
+        study_2.set_user_attr("key_2", "value_2")
+
+        # Run command again to include second study.
+        output = str(subprocess.check_output(command).decode().strip())
+        studies = _parse_output(output, output_format or "table")
+
         if output_format is None or output_format == "table":
             expected_keys_1 = expected_keys_2 = [
                 "name",
@@ -346,15 +425,17 @@ def test_studies_command_flatten(output_format: Optional[str]) -> None:
                 "direction_1",
                 "n_trials",
                 "datetime_start",
+                "user_attrs",
             ]
         else:
-            expected_keys_1 = ["name", "direction_0", "n_trials", "datetime_start"]
+            expected_keys_1 = ["name", "direction_0", "n_trials", "datetime_start", "user_attrs"]
             expected_keys_2 = [
                 "name",
                 "direction_0",
                 "direction_1",
                 "n_trials",
                 "datetime_start",
+                "user_attrs",
             ]
 
         assert len(studies) == 2
@@ -365,24 +446,29 @@ def test_studies_command_flatten(output_format: Optional[str]) -> None:
             assert set(studies[0].keys()) == set(expected_keys_1)
             assert set(studies[1].keys()) == set(expected_keys_2)
 
-        # Check study_name, direction, and n_trials for the first study.
+        # Check study_name, direction, n_trials and user_attrs for the first study.
         assert studies[0]["name"] == study_1.study_name
         if output_format is None or output_format == "table":
             assert studies[0]["n_trials"] == "0"
+            assert studies[0]["user_attrs"] == "{}"
         else:
             assert studies[0]["n_trials"] == 0
+            assert studies[0]["user_attrs"] == {}
         assert studies[0]["direction_0"] == "MINIMIZE"
 
-        # Check study_name, direction, and n_trials for the second study.
+        # Check study_name, direction, n_trials and user_attrs for the second study.
         assert studies[1]["name"] == study_2.study_name
         if output_format is None or output_format == "table":
             assert studies[1]["n_trials"] == "10"
+            assert studies[1]["user_attrs"] == "{'key_1': 'value_1', 'key_2': 'value_2'}"
         else:
             assert studies[1]["n_trials"] == 10
+            assert studies[1]["user_attrs"] == {"key_1": "value_1", "key_2": "value_2"}
         assert studies[1]["direction_0"] == "MINIMIZE"
         assert studies[1]["direction_1"] == "MAXIMIZE"
 
 
+@pytest.mark.skip_coverage
 @pytest.mark.parametrize("objective", (objective_func, objective_func_branched_search_space))
 @pytest.mark.parametrize("output_format", (None, "table", "json", "yaml"))
 def test_trials_command(objective: Callable[[Trial], float], output_format: Optional[str]) -> None:
@@ -393,7 +479,7 @@ def test_trials_command(objective: Callable[[Trial], float], output_format: Opti
         study_name = "test_study"
         n_trials = 10
 
-        study = optuna.create_study(storage, study_name=study_name)
+        study = optuna.create_study(storage=storage, study_name=study_name)
         study.optimize(objective, n_trials=n_trials)
         attrs = (
             "number",
@@ -463,6 +549,7 @@ def test_trials_command(objective: Callable[[Trial], float], output_format: Opti
                     assert value == str(expected_value)
 
 
+@pytest.mark.skip_coverage
 @pytest.mark.parametrize("objective", (objective_func, objective_func_branched_search_space))
 @pytest.mark.parametrize("output_format", (None, "table", "json", "yaml"))
 def test_trials_command_flatten(
@@ -475,7 +562,7 @@ def test_trials_command_flatten(
         study_name = "test_study"
         n_trials = 10
 
-        study = optuna.create_study(storage, study_name=study_name)
+        study = optuna.create_study(storage=storage, study_name=study_name)
         study.optimize(objective, n_trials=n_trials)
         attrs = (
             "number",
@@ -541,6 +628,7 @@ def test_trials_command_flatten(
                     assert value == str(expected_value)
 
 
+@pytest.mark.skip_coverage
 @pytest.mark.parametrize("objective", (objective_func, objective_func_branched_search_space))
 @pytest.mark.parametrize("output_format", (None, "table", "json", "yaml"))
 def test_best_trial_command(
@@ -553,7 +641,7 @@ def test_best_trial_command(
         study_name = "test_study"
         n_trials = 10
 
-        study = optuna.create_study(storage, study_name=study_name)
+        study = optuna.create_study(storage=storage, study_name=study_name)
         study.optimize(objective, n_trials=n_trials)
         attrs = (
             "number",
@@ -624,6 +712,7 @@ def test_best_trial_command(
                 assert value == str(expected_value)
 
 
+@pytest.mark.skip_coverage
 @pytest.mark.parametrize("objective", (objective_func, objective_func_branched_search_space))
 @pytest.mark.parametrize("output_format", (None, "table", "json", "yaml"))
 def test_best_trial_command_flatten(
@@ -636,7 +725,7 @@ def test_best_trial_command_flatten(
         study_name = "test_study"
         n_trials = 10
 
-        study = optuna.create_study(storage, study_name=study_name)
+        study = optuna.create_study(storage=storage, study_name=study_name)
         study.optimize(objective, n_trials=n_trials)
         attrs = (
             "number",
@@ -702,6 +791,7 @@ def test_best_trial_command_flatten(
                 assert value == str(expected_value)
 
 
+@pytest.mark.skip_coverage
 @pytest.mark.parametrize("output_format", (None, "table", "json", "yaml"))
 def test_best_trials_command(output_format: Optional[str]) -> None:
 
@@ -712,7 +802,7 @@ def test_best_trials_command(output_format: Optional[str]) -> None:
         n_trials = 10
 
         study = optuna.create_study(
-            storage, study_name=study_name, directions=("minimize", "minimize")
+            storage=storage, study_name=study_name, directions=("minimize", "minimize")
         )
         study.optimize(objective_func_multi_objective, n_trials=n_trials)
         attrs = (
@@ -786,6 +876,7 @@ def test_best_trials_command(output_format: Optional[str]) -> None:
                     assert value == str(expected_value)
 
 
+@pytest.mark.skip_coverage
 @pytest.mark.parametrize("output_format", (None, "table", "json", "yaml"))
 def test_best_trials_command_flatten(output_format: Optional[str]) -> None:
 
@@ -796,7 +887,7 @@ def test_best_trials_command_flatten(output_format: Optional[str]) -> None:
         n_trials = 10
 
         study = optuna.create_study(
-            storage, study_name=study_name, directions=("minimize", "minimize")
+            storage=storage, study_name=study_name, directions=("minimize", "minimize")
         )
         study.optimize(objective_func_multi_objective, n_trials=n_trials)
         attrs = (
@@ -864,6 +955,7 @@ def test_best_trials_command_flatten(output_format: Optional[str]) -> None:
                     assert value == str(expected_value)
 
 
+@pytest.mark.skip_coverage
 def test_create_study_command_with_skip_if_exists() -> None:
 
     with StorageSupplier("sqlite") as storage:
@@ -899,67 +991,16 @@ def test_create_study_command_with_skip_if_exists() -> None:
         assert study_id == new_study_id  # The existing study instance is reused.
 
 
-def test_dashboard_command() -> None:
-
-    with StorageSupplier("sqlite") as storage, tempfile.NamedTemporaryFile("r") as tf_report:
-        assert isinstance(storage, RDBStorage)
-        storage_url = str(storage.engine.url)
-
-        study_name = storage.get_study_name_from_id(storage.create_new_study())
-
-        command = [
-            "optuna",
-            "dashboard",
-            "--study-name",
-            study_name,
-            "--out",
-            tf_report.name,
-            "--storage",
-            storage_url,
-        ]
-        subprocess.check_call(command)
-
-        html = tf_report.read()
-        assert "<body>" in html
-        assert "bokeh" in html
-
-
-@pytest.mark.parametrize(
-    "origins", [["192.168.111.1:5006"], ["192.168.111.1:5006", "192.168.111.2:5006"]]
-)
-def test_dashboard_command_with_allow_websocket_origin(origins: List[str]) -> None:
-
-    with StorageSupplier("sqlite") as storage, tempfile.NamedTemporaryFile("r") as tf_report:
-        assert isinstance(storage, RDBStorage)
-        storage_url = str(storage.engine.url)
-
-        study_name = storage.get_study_name_from_id(storage.create_new_study())
-        command = [
-            "optuna",
-            "dashboard",
-            "--study-name",
-            study_name,
-            "--out",
-            tf_report.name,
-            "--storage",
-            storage_url,
-        ]
-        for origin in origins:
-            command.extend(["--allow-websocket-origin", origin])
-        subprocess.check_call(command)
-
-        html = tf_report.read()
-        assert "<body>" in html
-        assert "bokeh" in html
-
-
+@pytest.mark.skip_coverage
 def test_study_optimize_command() -> None:
 
     with StorageSupplier("sqlite") as storage:
         assert isinstance(storage, RDBStorage)
         storage_url = str(storage.engine.url)
 
-        study_name = storage.get_study_name_from_id(storage.create_new_study())
+        study_name = storage.get_study_name_from_id(
+            storage.create_new_study(directions=[StudyDirection.MINIMIZE])
+        )
         command = [
             "optuna",
             "study",
@@ -985,6 +1026,7 @@ def test_study_optimize_command() -> None:
         )
 
 
+@pytest.mark.skip_coverage
 def test_study_optimize_command_inconsistent_args() -> None:
 
     with tempfile.NamedTemporaryFile() as tf:
@@ -1007,6 +1049,7 @@ def test_study_optimize_command_inconsistent_args() -> None:
             )
 
 
+@pytest.mark.skip_coverage
 def test_empty_argv() -> None:
 
     command_empty = ["optuna"]
@@ -1023,10 +1066,15 @@ def test_check_storage_url() -> None:
     storage_in_args = "sqlite:///args.db"
     assert storage_in_args == optuna.cli._check_storage_url(storage_in_args)
 
+    with pytest.warns(ExperimentalWarning):
+        with patch.dict("optuna.cli.os.environ", {"OPTUNA_STORAGE": "sqlite:///args.db"}):
+            optuna.cli._check_storage_url(None)
+
     with pytest.raises(CLIUsageError):
         optuna.cli._check_storage_url(None)
 
 
+@pytest.mark.skip_coverage
 def test_storage_upgrade_command() -> None:
 
     with StorageSupplier("sqlite") as storage:
@@ -1035,12 +1083,16 @@ def test_storage_upgrade_command() -> None:
 
         command = ["optuna", "storage", "upgrade"]
         with pytest.raises(CalledProcessError):
-            subprocess.check_call(command)
+            subprocess.check_call(
+                command,
+                env={k: v for k, v in os.environ.items() if k != "OPTUNA_STORAGE"},
+            )
 
         command.extend(["--storage", storage_url])
         subprocess.check_call(command)
 
 
+@pytest.mark.skip_coverage
 @pytest.mark.parametrize(
     "direction,directions,sampler,sampler_kwargs,output_format",
     [
@@ -1063,7 +1115,7 @@ def test_ask(
 
     study_name = "test_study"
     search_space = (
-        '{"x": {"name": "UniformDistribution", "attributes": {"low": 0.0, "high": 1.0}}, '
+        '{"x": {"name": "FloatDistribution", "attributes": {"low": 0.0, "high": 1.0}}, '
         '"y": {"name": "CategoricalDistribution", "attributes": {"choices": ["foo"]}}}'
     )
 
@@ -1092,7 +1144,8 @@ def test_ask(
         if output_format is not None:
             args += ["--format", output_format]
 
-        output = str(subprocess.check_output(args).decode().strip())
+        result = subprocess.run(args, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
+        output = str(result.stdout.decode().strip())
         trial = _parse_output(output, output_format or "json")
 
         if output_format == "table":
@@ -1108,7 +1161,12 @@ def test_ask(
             assert 0 <= trial["params"]["x"] <= 1
             assert trial["params"]["y"] == "foo"
 
+        if direction is not None or directions is not None:
+            warning_message = result.stderr.decode()
+            assert "FutureWarning" in warning_message
 
+
+@pytest.mark.skip_coverage
 @pytest.mark.parametrize(
     "direction,directions,sampler,sampler_kwargs,output_format",
     [
@@ -1131,7 +1189,7 @@ def test_ask_flatten(
 
     study_name = "test_study"
     search_space = (
-        '{"x": {"name": "UniformDistribution", "attributes": {"low": 0.0, "high": 1.0}}, '
+        '{"x": {"name": "FloatDistribution", "attributes": {"low": 0.0, "high": 1.0}}, '
         '"y": {"name": "CategoricalDistribution", "attributes": {"choices": ["foo"]}}}'
     )
 
@@ -1161,7 +1219,8 @@ def test_ask_flatten(
         if output_format is not None:
             args += ["--format", output_format]
 
-        output = str(subprocess.check_output(args).decode().strip())
+        result = subprocess.run(args, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
+        output = str(result.stdout.decode().strip())
         trial = _parse_output(output, output_format or "json")
 
         if output_format == "table":
@@ -1175,7 +1234,12 @@ def test_ask_flatten(
             assert 0 <= trial["params_x"] <= 1
             assert trial["params_y"] == "foo"
 
+        if direction is not None or directions is not None:
+            warning_message = result.stderr.decode()
+            assert "FutureWarning" in warning_message
 
+
+@pytest.mark.skip_coverage
 @pytest.mark.parametrize("output_format", (None, "table", "json", "yaml"))
 def test_ask_empty_search_space(output_format: str) -> None:
     study_name = "test_study"
@@ -1208,6 +1272,7 @@ def test_ask_empty_search_space(output_format: str) -> None:
             assert trial["params"] == {}
 
 
+@pytest.mark.skip_coverage
 @pytest.mark.parametrize("output_format", (None, "table", "json", "yaml"))
 def test_ask_empty_search_space_flatten(output_format: str) -> None:
     study_name = "test_study"
@@ -1241,6 +1306,207 @@ def test_ask_empty_search_space_flatten(output_format: str) -> None:
             assert "params" not in trial
 
 
+@pytest.mark.skip_coverage
+def test_ask_sampler_kwargs_without_sampler() -> None:
+
+    study_name = "test_study"
+    search_space = (
+        '{"x": {"name": "FloatDistribution", "attributes": {"low": 0.0, "high": 1.0}}, '
+        '"y": {"name": "CategoricalDistribution", "attributes": {"choices": ["foo"]}}}'
+    )
+
+    with tempfile.NamedTemporaryFile() as tf:
+        db_url = "sqlite:///{}".format(tf.name)
+
+        args = [
+            "optuna",
+            "ask",
+            "--storage",
+            db_url,
+            "--study-name",
+            study_name,
+            "--search-space",
+            search_space,
+            "--sampler-kwargs",
+            '{"multivariate": true}',
+        ]
+
+        result = subprocess.run(args, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
+        error_message = result.stderr.decode()
+        assert "`--sampler_kwargs` is set without `--sampler`." in error_message
+
+
+@pytest.mark.skip_coverage
+@pytest.mark.parametrize(
+    "direction,directions,sampler,sampler_kwargs",
+    [
+        (None, None, None, None),
+        ("minimize", None, None, None),
+        (None, "minimize maximize", None, None),
+        (None, None, "RandomSampler", None),
+        (None, None, "TPESampler", '{"multivariate": true}'),
+    ],
+)
+def test_create_study_and_ask(
+    direction: Optional[str],
+    directions: Optional[str],
+    sampler: Optional[str],
+    sampler_kwargs: Optional[str],
+) -> None:
+
+    study_name = "test_study"
+    search_space = (
+        '{"x": {"name": "FloatDistribution", "attributes": {"low": 0.0, "high": 1.0}}, '
+        '"y": {"name": "CategoricalDistribution", "attributes": {"choices": ["foo"]}}}'
+    )
+
+    with tempfile.NamedTemporaryFile() as tf:
+        db_url = "sqlite:///{}".format(tf.name)
+
+        create_study_args = [
+            "optuna",
+            "create-study",
+            "--storage",
+            db_url,
+            "--study-name",
+            study_name,
+        ]
+
+        if direction is not None:
+            create_study_args += ["--direction", direction]
+        if directions is not None:
+            create_study_args += ["--directions"] + directions.split()
+        subprocess.check_call(create_study_args)
+
+        args = [
+            "optuna",
+            "ask",
+            "--storage",
+            db_url,
+            "--study-name",
+            study_name,
+            "--search-space",
+            search_space,
+        ]
+
+        if sampler is not None:
+            args += ["--sampler", sampler]
+        if sampler_kwargs is not None:
+            args += ["--sampler-kwargs", sampler_kwargs]
+
+        output = str(subprocess.check_output(args).decode().strip())
+        trial = _parse_output(output, "json")
+
+        assert trial["number"] == 0
+        assert 0 <= trial["params"]["x"] <= 1
+        assert trial["params"]["y"] == "foo"
+
+
+@pytest.mark.skip_coverage
+@pytest.mark.parametrize(
+    "direction,directions,ask_direction,ask_directions",
+    [
+        (None, None, "maximize", None),
+        ("minimize", None, "maximize", None),
+        ("minimize", None, None, "minimize minimize"),
+        (None, "minimize maximize", None, "maximize minimize"),
+        (None, "minimize maximize", "minimize", None),
+    ],
+)
+def test_create_study_and_ask_with_inconsistent_directions(
+    direction: Optional[str],
+    directions: Optional[str],
+    ask_direction: Optional[str],
+    ask_directions: Optional[str],
+) -> None:
+
+    study_name = "test_study"
+    search_space = (
+        '{"x": {"name": "FloatDistribution", "attributes": {"low": 0.0, "high": 1.0}}, '
+        '"y": {"name": "CategoricalDistribution", "attributes": {"choices": ["foo"]}}}'
+    )
+
+    with tempfile.NamedTemporaryFile() as tf:
+        db_url = "sqlite:///{}".format(tf.name)
+
+        create_study_args = [
+            "optuna",
+            "create-study",
+            "--storage",
+            db_url,
+            "--study-name",
+            study_name,
+        ]
+
+        if direction is not None:
+            create_study_args += ["--direction", direction]
+        if directions is not None:
+            create_study_args += ["--directions"] + directions.split()
+        subprocess.check_call(create_study_args)
+
+        args = [
+            "optuna",
+            "ask",
+            "--storage",
+            db_url,
+            "--study-name",
+            study_name,
+            "--search-space",
+            search_space,
+        ]
+        if ask_direction is not None:
+            args += ["--direction", ask_direction]
+        if ask_directions is not None:
+            args += ["--directions"] + ask_directions.split()
+
+        result = subprocess.run(args, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
+        error_message = result.stderr.decode()
+        assert "Cannot overwrite study direction" in error_message
+
+
+@pytest.mark.skip_coverage
+def test_ask_with_both_direction_and_directions() -> None:
+
+    study_name = "test_study"
+    search_space = (
+        '{"x": {"name": "FloatDistribution", "attributes": {"low": 0.0, "high": 1.0}}, '
+        '"y": {"name": "CategoricalDistribution", "attributes": {"choices": ["foo"]}}}'
+    )
+
+    with tempfile.NamedTemporaryFile() as tf:
+        db_url = "sqlite:///{}".format(tf.name)
+
+        create_study_args = [
+            "optuna",
+            "create-study",
+            "--storage",
+            db_url,
+            "--study-name",
+            study_name,
+        ]
+        subprocess.check_call(create_study_args)
+
+        args = [
+            "optuna",
+            "ask",
+            "--storage",
+            db_url,
+            "--study-name",
+            study_name,
+            "--search-space",
+            search_space,
+            "--direction",
+            "minimize",
+            "--directions",
+            "minimize",
+        ]
+
+        result = subprocess.run(args, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
+        error_message = result.stderr.decode()
+        assert "Specify only one of `direction` and `directions`." in error_message
+
+
+@pytest.mark.skip_coverage
 def test_tell() -> None:
     study_name = "test_study"
 
@@ -1263,7 +1529,7 @@ def test_tell() -> None:
         output = json.loads(output)
         trial_number = output["number"]
 
-        output = subprocess.check_output(
+        subprocess.check_output(
             [
                 "optuna",
                 "tell",
@@ -1280,3 +1546,103 @@ def test_tell() -> None:
         assert len(study.trials) == 1
         assert study.trials[0].state == TrialState.COMPLETE
         assert study.trials[0].values == [1.2]
+
+        # Error when updating a finished trial.
+        ret = subprocess.run(
+            [
+                "optuna",
+                "tell",
+                "--storage",
+                db_url,
+                "--trial-number",
+                str(trial_number),
+                "--values",
+                "1.2",
+            ]
+        )
+        assert ret.returncode != 0
+
+        # Passing `--skip-if-finished` to a finished trial for a noop.
+        subprocess.check_output(
+            [
+                "optuna",
+                "tell",
+                "--storage",
+                db_url,
+                "--trial-number",
+                str(trial_number),
+                "--values",
+                "1.3",  # Setting a different value and make sure it's not persisted.
+                "--skip-if-finished",
+            ]
+        )
+
+        study = optuna.load_study(storage=db_url, study_name=study_name)
+        assert len(study.trials) == 1
+        assert study.trials[0].state == TrialState.COMPLETE
+        assert study.trials[0].values == [1.2]
+
+
+@pytest.mark.skip_coverage
+def test_tell_with_nan() -> None:
+    study_name = "test_study"
+
+    with tempfile.NamedTemporaryFile() as tf:
+        db_url = "sqlite:///{}".format(tf.name)
+
+        output: Any = subprocess.check_output(
+            [
+                "optuna",
+                "ask",
+                "--storage",
+                db_url,
+                "--study-name",
+                study_name,
+                "--format",
+                "json",
+            ]
+        )
+        output = output.decode("utf-8")
+        output = json.loads(output)
+        trial_number = output["number"]
+
+        subprocess.check_output(
+            [
+                "optuna",
+                "tell",
+                "--storage",
+                db_url,
+                "--trial-number",
+                str(trial_number),
+                "--values",
+                "nan",
+            ]
+        )
+
+        study = optuna.load_study(storage=db_url, study_name=study_name)
+        assert len(study.trials) == 1
+        assert study.trials[0].state == TrialState.FAIL
+        assert study.trials[0].values is None
+
+
+@pytest.mark.skip_coverage
+@pytest.mark.parametrize(
+    "verbosity, expected",
+    [
+        ("--verbose", True),
+        ("--quiet", False),
+    ],
+)
+def test_configure_logging_verbosity(verbosity: str, expected: bool) -> None:
+
+    with StorageSupplier("sqlite") as storage:
+        assert isinstance(storage, RDBStorage)
+        storage_url = str(storage.engine.url)
+
+        # Create study.
+        args = ["optuna", "create-study", "--storage", storage_url, verbosity]
+        # `--verbose` makes the log level DEBUG.
+        # `--quiet` makes the log level WARNING.
+        result = subprocess.run(args, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
+        error_message = result.stderr.decode()
+        assert ("A new study created in RDB with name" in error_message) == expected
