@@ -1,5 +1,5 @@
 """
-.. _sampler:
+.. _user_defined_sampler:
 
 User-Defined Sampler
 ====================
@@ -17,7 +17,7 @@ Overview of Sampler
 -------------------
 
 A sampler has the responsibility to determine the parameter values to be evaluated in a trial.
-When a `suggest` API (e.g., :func:`~optuna.trial.Trial.suggest_float`) is called inside an objective function, the corresponding distribution object (e.g., :class:`~optuna.distributions.UniformDistribution`) is created internally. A sampler samples a parameter value from the distribution. The sampled value is returned to the caller of the `suggest` API and evaluated in the objective function.
+When a `suggest` API (e.g., :func:`~optuna.trial.Trial.suggest_float`) is called inside an objective function, the corresponding distribution object (e.g., :class:`~optuna.distributions.FloatDistribution`) is created internally. A sampler samples a parameter value from the distribution. The sampled value is returned to the caller of the `suggest` API and evaluated in the objective function.
 
 To create a new sampler, you need to define a class that inherits :class:`~optuna.samplers.BaseSampler`.
 The base class has three abstract methods;
@@ -74,8 +74,16 @@ class SimulatedAnnealingSampler(optuna.samplers.BaseSampler):
         # the objective function passed to the study.
         params = {}
         for param_name, param_distribution in search_space.items():
-            if not isinstance(param_distribution, optuna.distributions.UniformDistribution):
-                raise NotImplementedError("Only suggest_float() is supported")
+            if (
+                not isinstance(param_distribution, optuna.distributions.FloatDistribution)
+                or (param_distribution.step is not None and param_distribution.step != 1)
+                or param_distribution.log
+            ):
+                msg = (
+                    "Only suggest_float() with `step` `None` or 1.0 and"
+                    " `log` `False` is supported"
+                )
+                raise NotImplementedError(msg)
 
             current_value = self._current_trial.params[param_name]
             width = (param_distribution.high - param_distribution.low) * 0.1
@@ -108,7 +116,7 @@ class SimulatedAnnealingSampler(optuna.samplers.BaseSampler):
 def objective(trial):
     x = trial.suggest_float("x", -10, 10)
     y = trial.suggest_float("y", -5, 5)
-    return x ** 2 + y
+    return x**2 + y
 
 
 sampler = SimulatedAnnealingSampler()
